#include <utility.h>
#include <userint.h>

/*****************************************************************************/
/*   Copyright 1998 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/


/*================ R3267/73 Spectrum Analyzer OPT62(3GPP)====================*/
/* LabWindows/CVI 5.0 Instrument Driver										 */
/* Original Release: October/17/2000										 */
/* By: Ai Onishi, Hitoshi Takahashi											 */
/*					 Supervisor: Yohei Hirakoso								 */
/*                   SPSpj Gyoda R&D,  ADVANTEST CORPORATION				 */
/*                   Home Page; http://www.advantest.co.jp					 */
/*                   e-mail;    info@advantest.co.jp						 */
/*---------------------------------------------------------------------------*/
/* Modification History: 													 */
/*         Version 1.0 :  October/17/2000									 */
/*																			 */
/*===========================================================================*/
 

#include <visa.h>
#include <ansi_c.h>
#include <formatio.h>
#include "avopt62.h"

#define avopt62_REVISION    "Rev 1.0, 09/00, CVI 5.0" /*  Instrument driver revision */
#define BUFFER_SIZE         1024L 					  /* File I/O buffer size */
    

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

/****
 ****  Opt62 Static Command Arr
 ****/

/* 3GPP Started Setting */
static ViString measModeArr[] = {"MEASMD FRAME","MEASMD SLOT"};
static ViString linkArr[] = {"LINK DOWN","LINK UP"};
static ViString signalRateArr[] = {"RATE 9600","RATE 4800","RATE 2400","RATE 1200"};
static ViString frequencyInputArr[]={"FINPMD CHL","FINPMD FREQ"};
static ViString inputArr[]={"INPUT RF","INPUT IQ"};
static ViString baseBandInputArr[]={"BBINPUT AC","BBINPUT DC"};
static ViString autoLevelArr[] = {"ALS OFF","ALS ON"}; 

/* Channel Setting */
static ViString edtChArr[]={"CHEDUP1","CHEDDN1","CHEDUP2","CHEDDN2","CHEDUP3","CHEDDN3"};

/* Channel Enable */
static ViString enableArr[]={"DSBL","ENBL"};     

/* Trigger */
static ViString trgWFSrcArr[] = {"TRGSRC FREE","TRGSRC VIDEO","TRGSRC IF","TRGSRC EXT"};
static ViString trgSlopeArr[] = {"TRGSLP FALL","TRGSLP RISE"};

/* Gate */
static ViString trgFdpSrcArr[] = {"FREE","VIDEO","IF","EXT"};
static ViString gateExSTDArr[] = {"DTMSETSTD","FDPSETSTD"};

/* Window set */
static ViString switchArr[] = {"OFF","ON"};       

/* Template Edit */
static ViString UpLowArr[]={"LOW","UP"};

/* Detecter Select */
static ViString DetArr[]={"DET NRM","DET POS","DET NEG","DET SMP"};

/* Select Table 1 to 3 */
static ViString selTblArr[] = {"TBL 1","TBL 2","TBL 3"};

/* Table Frequency Type */   
static ViString tableFArr[] = {"TBLF REL","TBLF ABS"};

/*TDS Param*/
static ViString resultArr[] = {"RES RMS","RES PK"};
static ViString preselArr[]={"PRE 36G", "PRE 16G"};

/* DTS Param */
static ViString FreqSetArr[] ={"FRMD CFSP","FRMD STSP"};
static ViString LvlUnitArr[] ={"UNIT DBM", "UNIT W", "UNIT DBUV"};
static ViString dtsResultArr[] = {"RES MKR", "RES REL","RES ABS"};
static ViString refMkrArr[] = {"REF MOD","REF MKR"};

/* Modulation */
static ViString codeDefArr[] = {"3GPSCDEF UNDEF", "3GPSCDEF DEF"};
static ViString trgModeArr[] = {"TRG EXT", "TRG INT"};
static ViString srchModeArr[] = {"3GPSRCH PCPICH", "3GPSRCH SCH"};
static ViString actCHDtctArr[] = {"3GPACTCH AUTO", "3GPACTCH USER",
								  "3GPACTCH T1DP16", "3GPACTCH T1DP32", "3GPACTCH T1DP64",
								  "3GPACTCH T2", "3GPACTCH T3DP16", "3GPACTCH T3DP32"};
static ViString downAnaRateArr[] = {"3GPDNRATE ACT", "3GPDNRATE 7K5", "3GPDNRATE 15K",
									"3GPDNRATE 30K", "3GPDNRATE 60K", "3GPDNRATE 120K",
									"3GPDNRATE 240K", "3GPDNRATE 480K", "3GPDNRATE 960K"};
static ViString upAnaRateArr[] = {"3GPUPRATE 15K", "3GPUPRATE 30K", "3GPUPRATE 60K",
								  "3GPUPRATE 120K", "3GPUPRATE 240K", "3GPUPRATE 480K",
								  "3GPUPRATE 960K"};
static ViString phsInvArr[] = {"PHASE INV", "PHASE NORM"};
static ViString freqErrArr[] = {"FERR PREC", "FERR NORM"};
static ViString msBranchArr[] = {"3GPMBRCH Q", "3GPMBRCH I"};
static ViString pwrUnitArr[] = {"UNIT ABS", "UNIT REL"};
static ViString pwrtmTrgModeArr[] = {"3GPPTTRG INT", "3GPPTTRG IF", "3GPPTTRG EXT"};
static ViString grPltTypArr[] = {"3GPPTGTYP PK", "3GPPTGTYP AVG"};

/* Tx Power */
static ViString TXtrigSourceArr[] = {"TTRGSRC INT","TTRGSRC EXT","TTRGSRC INTRVL1"};

/* Avarage */
static ViString avgFunctionArr[] = {"TDPAVG", "OORAVG", "TDSAVG", "FDPAVG", "OBWAVG",
									"DTSAVG", "DTMAVG", "SPRAVG", "FDSAVG",
									"3GPAVG", "QPAVG", "TXAVG"};
/* Auto Level */
static ViString autoLvlFunctionArr[] = {"TDPAUTOLVL","OORAUTOLVL","TDSAUTOLVL","FDPAUTOLVL",
										"OBWAUTOLVL", "DTSAUTOLVL", "DTMAUTOLVL", "SPRAUTOLVL",
										"FDSAUTOLVL", "AUTOLVL"};

/* Mesurement Start */
static ViString funcMeasureArr[] = {"TDPMEAS", "OORMEAS", "TDSMEAS", "FDPMEAS", "OBWMEAS", "DTSMEAS",
									"DTMMEAS", "SPRMEAS", "FDSMEAS",
									"3GPP", "QPSK", "TXPWR", "3GPPT", "3GPCCDF", "SI"};

/* Graph Type */
static ViString grphTyp3GPPArr[] = {"3GPGTYP CON", "3GPGTYP CONLIN", "3GPGTYP CONDOT", 
									"3GPGTYP CONLINDOT", "3GPGTYP ICHEYE", "3GPGTYP QCHEYE",
									"3GPGTYP IQCHEYE", "3GPGTYP EVM", "3GPGTYP MAGERR",
									"3GPGTYP PHAERR", "3GPGTYP SCHPWR"};
static ViString grphTypQPSKArr[] = {"QPGTYP CON", "QPGTYP CONLIN", "QPGTYP CONDOT", 
									"QPGTYP CONLINCHP", "QPGTYP ICHEYE", "QPGTYP QCHEYE",
									"QPGTYP IQCHEYE", "QPGTYP EVM", "QPGTYP MAGERR",
									"QPGTYP PHAERR"};
								  
/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* avopt62_stringValPair is used in the avopt62_errorMessage function        */
/* avopt62_statusDataRanges is used to track session dependent status & ranges*/
/*===========================================================================*/
typedef struct  avopt62_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  avopt62_tStringValPair;
 
struct avopt62_statusDataRanges {
    ViInt16 traceRange;
    ViInt16 harmonics;
    ViChar instrDriverRevision[256];
};

typedef struct avopt62_statusDataRanges *avopt62_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean avopt62_invalidViBooleanRange (ViBoolean val);
ViBoolean avopt62_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean avopt62_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean avopt62_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max);
ViBoolean avopt62_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean avopt62_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean avopt62_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean avopt62_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus avopt62_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn,
	ViUInt32 tmoIn, ViPUInt16 STB);
ViStatus avopt62_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
	ViStatus currentStatus);
ViStatus avopt62_defaultInstrSetup (ViSession openInstrSession);
ViStatus avopt62_dataGraph (ViSession instrumentHandle, ViInt16 grphSlct,
                            ViInt16 *iDataSize, ViReal64 iData[],
                            ViInt16 *qDataSize, ViReal64 qData[],
                            ViInt16 *xDataSize, ViInt16  xData[],
                            ViInt16 *yDataSize, ViReal64 yData[]);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Standard Setting                                                */
/* Purpose:  This function sets the communication type and link of the signal*/
/*           under TRANSIENT measurement.									 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_confStdSet (ViSession instrumentHandle, ViBoolean measMode,
                                      ViBoolean link, ViReal64 offsetLevel,
                                      ViBoolean frequencyInput, ViBoolean input,
                                      ViBoolean baseBandInput,
                                      ViBoolean contAutoLevel)
{
	ViUInt32 retCnt = 0;
	ViChar wrtBuf[BUFFER_SIZE];
	ViStatus avopt62_status = VI_SUCCESS;
    
	if (avopt62_invalidViBooleanRange (measMode))
		return VI_ERROR_PARAMETER2;
	if (avopt62_invalidViBooleanRange (link))
		return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViReal64Range (offsetLevel, -100.0, 100.0))
        return VI_ERROR_PARAMETER4;
	if (avopt62_invalidViBooleanRange (frequencyInput))
		return VI_ERROR_PARAMETER5;
	if (avopt62_invalidViBooleanRange (input))
		return VI_ERROR_PARAMETER6;
	if (avopt62_invalidViBooleanRange (baseBandInput))
		return VI_ERROR_PARAMETER7;
	if (avopt62_invalidViBooleanRange (contAutoLevel))
		return VI_ERROR_PARAMETER8;
    
    Fmt (wrtBuf, "%s<%s;%s;RO %fDB;%s;%s;%s;%s", measModeArr[measMode], linkArr[link],
    	 offsetLevel, frequencyInputArr[frequencyInput], inputArr[input], 
    	 baseBandInputArr[baseBandInput], autoLevelArr[contAutoLevel]);
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;
    	
}

/*===========================================================================*/
/* Function: CDMA Channel Setting                                            */
/* Purpose: This function edits channel setting.                             */
/*               						                                	 */
/*===========================================================================*/

ViStatus CVIFUNC avopt62_ChEdt (ViSession instrumentHandle, ViBoolean settoSTD,
                                ViInt16 editChannel, ViInt16 startCHNum,
                                ViInt16 stopCHNum, ViReal64 baseFreqHz,
                                ViReal64 channelSpaceHz, ViInt16 CHOffset)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViInt16Range (editChannel, 0, 5))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViInt16Range (startCHNum, -12800, 12800))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViInt16Range (stopCHNum, -12800, 12800))
        return VI_ERROR_PARAMETER5;
    if (avopt62_invalidViReal64Range (baseFreqHz, 0.0, 100000000000.0))
        return VI_ERROR_PARAMETER6;
    if (avopt62_invalidViReal64Range (channelSpaceHz, -100000000.0, 100000000.0))
        return VI_ERROR_PARAMETER7;
    if (avopt62_invalidViInt16Range (CHOffset, -12800, 12800))
        return VI_ERROR_PARAMETER8;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<CHSETSTD");
    	total += NumFmtdBytes();
	}
	else
	{
		Fmt (wrtBuf, "%s<%s %d,%d,%fHZ,%fHZ,%d", edtChArr[editChannel],
    	startCHNum,stopCHNum,baseFreqHz,channelSpaceHz,CHOffset);
    	total += NumFmtdBytes();
	}

    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;  
   
}


/*===========================================================================*/
/* Function: Channel Enable ON/OFF                                           */
/* Purpose:  This function sets the state of Channel table enable.           */
/*                                      									 */
/*===========================================================================*/

ViStatus CVIFUNC avopt62_ch_Enable (ViSession instrumentHandle,
                                    ViBoolean channel1, ViBoolean channel2,
                                    ViBoolean channel3)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViBooleanRange (channel1))
        return VI_ERROR_PARAMETER2;	
    if (avopt62_invalidViBooleanRange (channel2))
        return VI_ERROR_PARAMETER3;	   
    if (avopt62_invalidViBooleanRange (channel3))
        return VI_ERROR_PARAMETER4;	           


	Fmt (wrtBuf, "%s<CHTBL1 %s;CHTBL2 %s;CHTBL3 %s",enableArr[channel1],
		enableArr[channel2],enableArr[channel3]);
    total += NumFmtdBytes();


    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;      
}

/*===========================================================================*/
/* Function: TDP Trigger Conditions                                          */
/* Purpose:  This function is used for time domain Power. This               */
/*           function sets the trigger parameters.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_confTrgConTDP (ViSession instrumentHandle,
                                        ViInt16 triggerSource,
                                        ViBoolean trgSlope, 
                                        ViInt16 triggerLevel,
                                        ViInt16 triggerPosition,
                                        ViReal64 delayTimes)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViInt16Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;
    if (avopt62_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<%s;%s;TRGLVL %dENT", trgWFSrcArr[triggerSource],
    	trgSlopeArr[trgSlope], triggerLevel);
    total += NumFmtdBytes();
	if (triggerSource > 0) {
	    Fmt (wrtBuf, "%s[a]<;TRGPOS %dENT;TRGDT %f[e]SC", triggerPosition, delayTimes); /* Append */
	}
    total += NumFmtdBytes();
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;       
    
}

/*===========================================================================*/
/* Function: TDP Window Setup					                             */
/* Purpose:  This function is used for time domain Power.					 */
/*           This function sets the Window setup parameters.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_windowSetTDP (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViBoolean window,
                                       ViReal64 winPosS, ViReal64 winWidS)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    

    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViBooleanRange (window))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViReal64Range (winPosS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER4;   
    if (avopt62_invalidViReal64Range (winWidS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER5; 

    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<TDPSETSTD");
    	total += NumFmtdBytes();
    }
    else
    {
        Fmt (wrtBuf, "%s<TDPWDO %s; TDPWPOS %f[e]SC; TDPWWID %f[e]SC",
        	switchArr[window],
    		winPosS, winWidS);
    	total += NumFmtdBytes();
    }
               
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}


/*===========================================================================*/
/* Function: TDP Template Setup					                             */
/* Purpose:  This function is used for time domain Power.					 */
/*           This function sets the Template setup parameters.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_template_Set_TDP (ViSession instrumentHandle,
                                           ViBoolean template, ViReal64 shiftXS,
                                           ViReal64 shiftYdB)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    

	if (avopt62_invalidViBooleanRange (template))
        return VI_ERROR_PARAMETER2;

    if (avopt62_invalidViReal64Range (shiftXS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER3;   
    if (avopt62_invalidViReal64Range (shiftYdB, -200.0, 100.0))
        return VI_ERROR_PARAMETER4; 

    Fmt (wrtBuf, "%s<TDPTMPL %s; TDPTMPLSX %f[e]SC; TDPTMPLSY %fDB", switchArr[template],
    	shiftXS, shiftYdB);
    
    total += NumFmtdBytes();
    
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;      
}

/*===========================================================================*/
/* Function: TDP Template Edit					                             */
/* Purpose:  This function is used for Time domain Power.					 */
/*           This function sets the Table data of Template.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_template_editTDP (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean templateSelect,
                                           ViReal64 templateTimes,
                                           ViReal64 templateLevel)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    

	if (avopt62_invalidViBooleanRange (templateSelect))
        return VI_ERROR_PARAMETER3;
	if (avopt62_invalidViInt16Range (operation,0, 1))
        return VI_ERROR_PARAMETER2;
 
    Fmt (wrtBuf, "%s<TDPTMPLSEL %s", UpLowArr[templateSelect]);
	total += NumFmtdBytes();

	switch (operation)
	{
	case 0: /* entry */
		if (avopt62_invalidViReal64Range (templateTimes, -1.0, 1.0))
        	return VI_ERROR_PARAMETER4;

		Fmt (wrtBuf, "%s[a]<; TDPTMPLED %f[e]SC,%f", templateTimes, templateLevel);
		break;
		
	case 1: /* Table init */
		 Fmt(wrtBuf, "%s[a]<; TDPTMPLCLR"); 
		break;
	}
		total += NumFmtdBytes(); 
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: TDP Parameter Setup				                             */
/* Purpose:  This function is used for Time domain Power.					 */
/*           This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_parameter_setTDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean TMPLCoupleofPWR,
                                           ViReal64 templateLimit,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    

    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;


    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<TDPSETSTD");
    }
    else
    {
    	if (avopt62_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt62_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
    	if (avopt62_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_PARAMETER5;
   		if (avopt62_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER6;        
    
    
        Fmt (wrtBuf, "%s<TDP%s; TDP%s; TDPTMPLPW %s; TDPTMPLBTM %fDB; TDPJDG %s; TDPJDGUP %fDB; TDPJDGLOW %fDB",
        	DetArr[detector], LvlUnitArr[displayUnit], switchArr[TMPLCoupleofPWR],
        	templateLimit, switchArr[judgement], upperLimit,  lowerLimit);

    }
    	total += NumFmtdBytes();                 
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;        
}


/*===========================================================================*/
/* Function: On/Off Raito Trigger   				                         */
/* Purpose:  This function is used for On/Off Raito (Time Domain).			 */
/*           This function sets the parameters for Trigger. 				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_trigger_setOOR (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViInt16Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;
    if (avopt62_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<OOR%s; OOR%s; OORTRGLVL %dENT", trgWFSrcArr[triggerSource],
    	trgSlopeArr[trgSlope], triggerLevel);
    total += NumFmtdBytes();
    
	if (triggerSource > 0) {
	    Fmt (wrtBuf, "%s[a]<;OORTRGPOS %dENT; OORTRGDT %f[e]SC", triggerPosition, delayTimes);
    	total += NumFmtdBytes(); 
	}

    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;       
    	    
}

/*===========================================================================*/
/* Function: On/Off Raito Window						                     */
/* Purpose:  This function is used for time domain wave form analyzing. This */
/*           function sets the trigger parameters.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_window_setOOR (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 onPositionS, ViReal64 onWidthS,
                                        ViReal64 offPositionS,
                                        ViReal64 offWidthS)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    

    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViBooleanRange (window))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViReal64Range (onPositionS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER4;   
    if (avopt62_invalidViReal64Range (onWidthS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER5; 
    if (avopt62_invalidViReal64Range (offPositionS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER6;   
    if (avopt62_invalidViReal64Range (offWidthS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER7; 
        
    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<OORSETSTD");
    }
    else
    {
        Fmt (wrtBuf, "%s<OORWDO %s; OORWONPOS %f[e]SC; OORWONWID %f[e]SC; OORWOFPOS %f[e]SC; OORWOFWID %f[e]SC",
        	switchArr[window],onPositionS,onWidthS,offPositionS,offWidthS);
    }
    	total += NumFmtdBytes();           
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    	    
}

/*===========================================================================*/
/* Function: On/Off Raito Parameter Setup				                     */
/* Purpose:  This function is used for Time domain Power.					 */
/*           This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_param_setOOR (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViInt16 displayUnit, ViBoolean judgement,
                                       ViReal64 upperLimit)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    

    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;


    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<OORSETSTD");
    	total += NumFmtdBytes();
    }
    else
    {
    	if (avopt62_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt62_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
   		if (avopt62_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER5;        
    
    
        Fmt (wrtBuf, "%s<OOR%s; OOR%s; OORJDG %s; OORJDGUP %fDB",
        	DetArr[detector], LvlUnitArr[displayUnit], switchArr[judgement], upperLimit);
    	total += NumFmtdBytes();
    }
               
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}
/*===========================================================================*/
/* Function: TDS Trigger Set	                                             */
/* Purpose:  This function is used for time domain Spurious. This            */
/*           function sets the trigger parameters.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_trigger_setTDS (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViInt16Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;    
    if (avopt62_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<TDS%s;TDS%s", trgWFSrcArr[triggerSource],
    	trgSlopeArr[trgSlope]);
    total += NumFmtdBytes();
    
    
	if (triggerSource != 0 ) {
		
		if ((triggerSource == 1) || (triggerSource == 2)) {
	    	Fmt (wrtBuf, "%s[a]<;TDSTRGLVL %dENT", triggerLevel);
	    	total += NumFmtdBytes();		
		}
		
	    Fmt (wrtBuf, "%s[a]<;TDSTRGPOS %dENT;TDSTRGDT %f[e]SC", triggerPosition, delayTimes);
	    total += NumFmtdBytes();     
	}

    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;       
}

/*===========================================================================*/
/* Function: TDS Table Edit						                     		 */
/* Purpose:  This function is used for Time domain Spurious.				 */
/*           This function sets the Table for Measurement.					 */
/*===========================================================================*/

ViStatus CVIFUNC avopt62_table_editTDS (ViSession instrumentHandle,
                                        ViInt16 operation, ViInt16 tableNumber,
                                        ViBoolean freqType, ViReal64 carrierFreq,
                                        ViReal64 frequency, ViReal64 level)
{
    ViUInt32 i,retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
	if (avopt62_invalidViInt16Range (operation, 0, 6))
        return VI_ERROR_PARAMETER2;    
	if (avopt62_invalidViInt16Range (tableNumber, 1, 3))
        return VI_ERROR_PARAMETER3;

/*

	0: Load Table
	1: Save Table
	2: Select Table (Default value)
	3: Edit Table
	4: Delete Table
	5: Init Table
	6: Table Freq Type & Carrer Freq Input

*/

	switch (operation)
	{
	case 0: /* LOAD TABLE */
		Fmt (wrtBuf, "%s<RCL%s", selTblArr[tableNumber-1]);
    	total += NumFmtdBytes();
		break;
	case 1: /* SAVE TABLE */
		Fmt (wrtBuf, "%s<SVS%s", selTblArr[tableNumber-1]);
    	total += NumFmtdBytes();
		break;
	case 2: /* SELECT TABLE ONLY */
		Fmt (wrtBuf, "%s<TDS%s", selTblArr[tableNumber-1]);
    	total += NumFmtdBytes();
		break;
	case 3: /* DATA ENTRY */
		Fmt (wrtBuf, "%s<TDSTBLED %fHZ,%fDB",frequency, level);
    	total += NumFmtdBytes();
		break;
	case 4: /* DATA DELETE */
		Fmt (wrtBuf, "%s<TDSTBLDEL");
    	total += NumFmtdBytes();
		break;
	case 5: /* INIT TABLE */
		Fmt (wrtBuf, "%s<TDSCLR");
    	total += NumFmtdBytes();
		break;
	case 6: /* TABLE FREQ & CARRER FREQ */
		if (avopt62_invalidViBooleanRange (freqType))
        	return VI_ERROR_PARAMETER4;
        if (avopt62_invalidViReal64Range (carrierFreq, 0.0, 8000000000.0))
        	return VI_ERROR_PARAMETER5;	
		Fmt (wrtBuf, "%s<TDS%s; CF %fHZ",tableFArr[freqType],carrierFreq);
		break;
	}
	total += NumFmtdBytes();
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        	return avopt62_status;    
    return avopt62_status;     
}

/*===========================================================================*/
/* Function: TDS Parameter Setup				                     		 */
/* Purpose:  This function is used for Time domain Spurious.				 */
/*           This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_param_setTDS (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViBoolean result, ViReal64 peakMKRYDelta,
                                       ViReal64 multiplier, ViInt16 displayUnit,
                                       ViBoolean judgement, ViBoolean preselecter)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    

    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<TDSSETSTD");
    	total += NumFmtdBytes();
    }
    else
    {
    	if (avopt62_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
	    if (avopt62_invalidViBooleanRange (result))
        	return VI_ERROR_PARAMETER4;    
   		if (avopt62_invalidViReal64Range (peakMKRYDelta, 0.1, 10.0))
        	return VI_ERROR_PARAMETER5;
        if (avopt62_invalidViReal64Range (multiplier, 1.0, 100.0))
        	return VI_ERROR_PARAMETER6;
    	if (avopt62_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER7;
   		if (avopt62_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER8;        
           if (avopt62_invalidViBooleanRange (preselecter))
        return VI_ERROR_INSTR_PARAMETER9;
        
        Fmt (wrtBuf, "%s<TDS%s; TDS%s; TDSJDG %s; TDS%s; TDSPKMKY %fENT; TDSMULTI %fENT;TDS%s",
        	DetArr[detector], 
        	LvlUnitArr[displayUnit],
        	switchArr[judgement], 
        	resultArr[(ViInt16)result],
        	peakMKRYDelta, multiplier,preselArr[preselecter]);

    	total += NumFmtdBytes();
    }
               
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    	    
}

/*===========================================================================*/
/* Function: Gated Spectrum						                             */
/* Purpose:  This function is used for Freq domain wave form analyzing. This */
/*           function sets the Gate Setup parameters.	     				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_confGateTrg (ViSession instrumentHandle,
									  ViBoolean gateSource, ViBoolean settoSTD,
									  ViBoolean measMode, ViReal64 gatePosition,
									  ViReal64 gateWidth, ViInt16 trgSource, 
                                      ViInt16 triggerPosition, ViReal64 delayTime, 
                                      ViBoolean trgSlope, ViInt16 triggerLevel,
                                      ViBoolean gatedSweep, ViInt16 detector)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViBooleanRange (gateSource))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViBooleanRange (measMode))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViReal64Range (gatePosition, 0, 1))
        return VI_ERROR_PARAMETER5;
    if (avopt62_invalidViReal64Range (gateWidth, 0, 1))
        return VI_ERROR_PARAMETER6;
	if (avopt62_invalidViInt16Range (trgSource,0,3))
        return VI_ERROR_PARAMETER7;
    if (avopt62_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER8;
    if (avopt62_invalidViReal64Range (delayTime, -1000.0, 1000.0))
        return VI_ERROR_INSTR_PARAMETER9;
    if (avopt62_invalidViBooleanRange (trgSlope))
        return VI_ERROR_INSTR_PARAMETER10;
    if (avopt62_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_INSTR_PARAMETER11;
    if (avopt62_invalidViBooleanRange (gatedSweep))
        return VI_ERROR_INSTR_PARAMETER12;    
    if (avopt62_invalidViInt16Range (detector, 0, 3))
        return VI_ERROR_INSTR_PARAMETER13;

   	Fmt (wrtBuf, "%s<TGTSRC TRG; TGTTRG %s; TGT%s; TGTTRGLVL %d; TGTTRGPOS %d; TGTTRGDT %f[e]SC",
    	 trgFdpSrcArr[trgSource],
    	 trgSlopeArr[trgSlope], 
    	 triggerLevel, triggerPosition, delayTime);
    total += NumFmtdBytes();
	
   	if ( settoSTD == 1)          /* Set to STD: On */
    	Fmt (wrtBuf, "%s[a]<; TGTSETUP OFF; %s", gateExSTDArr[measMode]);
   		
   	else						 /* Set to STD: Off */
   		Fmt (wrtBuf, "%s[a]<; TGTPOS %f[e]SC; TGTWID %f[e]SC", gatePosition,gateWidth);
   	total += NumFmtdBytes();
   	
	if (gateSource == 0)      /* Gate Source: EXT */
	{
		Fmt (wrtBuf, "%s[a]<; TGTSRC EXT");
		total += NumFmtdBytes();
	}	

   	Fmt (wrtBuf, "%s[a]<; TGTSWP %s; TGT%s; TGTSETUP OFF", switchArr[gatedSweep],DetArr[detector]);
    total += NumFmtdBytes();
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: FDP Window Set						                             */
/* Purpose:  This function is used for Freq domain wave form analyzing. This */
/*           function sets the Gate Sweep Window.		     				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_window_setFDP (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 positionHz, ViReal64 widthHz)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<FDPSETSTD");	
	}
	else
	{
	    if (avopt62_invalidViBooleanRange (window))
        	return VI_ERROR_PARAMETER3;
	    if (avopt62_invalidViReal64Range (positionHz, 0.0, 8.0e+9))
        	return VI_ERROR_PARAMETER4;
    	if (avopt62_invalidViReal64Range (widthHz, 0.0, 8.0e+9))
        	return VI_ERROR_PARAMETER5;
        	
		Fmt (wrtBuf, "%s<FDPWDO %s;FDPWPOS %fHZ; FDPWWID %fHZ", 
			switchArr[window], positionHz, widthHz);
	}
        
        
    total += NumFmtdBytes();
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;

}

/*===========================================================================*/
/* Function: FDP Parameter Set					                             */
/* Purpose:  This function is used for Freq domain wave form analyzing. This */
/*           function sets the Gate Sweep Window.		     				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_parameter_setFDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean gatedSweep,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<FDPSETSTD");	
	    total += NumFmtdBytes();
	}
	else
	{
    	if (avopt62_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt62_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
		if (avopt62_invalidViBooleanRange (gatedSweep))
        	return VI_ERROR_PARAMETER5;     	
   		if (avopt62_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER6;        

        Fmt (wrtBuf, "%s<FDP%s; FDP%s;TGTSWP %s; FDPJDG %s; FDPJDGUP %f; FDPJDGLOW %f",
        	DetArr[detector], LvlUnitArr[displayUnit],
        	switchArr[gatedSweep], switchArr[judgement],
        	upperLimit,  lowerLimit);
    	total += NumFmtdBytes();	
	}
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;	
}

/*===========================================================================*/
/* Function: OBW Parameter Set                                               */
/* Purpose:  This function sets the parameters for OBW measurement.			 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_parameter_setOBW (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViReal64 OBW,
                                           ViInt16 detector, ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<OBWSETSTD");	
	    total += NumFmtdBytes();    
	
	}
	else
	{
	    if (avopt62_invalidViReal64Range (OBW, 0.5, 99.5))
        	return VI_ERROR_PARAMETER3;
    	if (avopt62_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER4;
   		if (avopt62_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER5;        
	    if (avopt62_invalidViReal64Range (upperLimit, 0, 100e+6))
        	return VI_ERROR_PARAMETER6;
	    if (avopt62_invalidViReal64Range (lowerLimit, 0, 100e+6))
        	return VI_ERROR_PARAMETER7;

        Fmt (wrtBuf, "%s<OBWPER %f; OBW%s; OBWJDG %s; OBWJDGUP %f; OBWJDGLOW %f",
        	OBW, DetArr[detector],
			switchArr[judgement], upperLimit, lowerLimit);
    	total += NumFmtdBytes();	
	}
        
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;		
}


/*===========================================================================*/
/* Function: DT Parameter Set                                                */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_parameter_setDT (ViSession instrumentHandle,
                                          ViBoolean settoSTD, ViInt16 detector,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViReal64 templateLimit,
                                          ViBoolean judgement,
                                          ViBoolean freqSetting, ViInt16 result,
                                          ViBoolean refPower,
                                          ViReal64 symbolRate, ViReal64 rolloff)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<DTSSETSTD");	
	    total += NumFmtdBytes();

	}
	else
	{
    	if (avopt62_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
   		if (avopt62_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
   		if (avopt62_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_PARAMETER5; 
   		if (avopt62_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER7;  
   		if (avopt62_invalidViBooleanRange (freqSetting))
        	return VI_ERROR_PARAMETER8;         	
    		if (avopt62_invalidViInt16Range (result, 0, 2))
        	return VI_ERROR_INSTR_PARAMETER9;
   		if (avopt62_invalidViBooleanRange (refPower))
        	return VI_ERROR_INSTR_PARAMETER10;
   		if (avopt62_invalidViReal64Range (symbolRate, 1.0, 1.0e+9))
        	return VI_ERROR_INSTR_PARAMETER11;
   		if (avopt62_invalidViReal64Range (rolloff, 0.01, 0.99))
        	return VI_ERROR_INSTR_PARAMETER12;
        
        Fmt (wrtBuf, "%s<DTS%s; DTS%s; DTS%s; DTS%s; DTS%s;DTSTMPLPW %s;DTSTMPLBTM %fDB; DTSJDG %s; DTSSYMRT %fHZ; DTSRFACT %f",
        	FreqSetArr[freqSetting], DetArr[detector], dtsResultArr[result],
        	 refMkrArr[refPower], LvlUnitArr[displayUnit],switchArr[TMPLCoupleofPWR],
			templateLimit, switchArr[judgement], symbolRate, rolloff);
    	total += NumFmtdBytes();	
	}
        
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;	    
}

/*===========================================================================*/
/* Function: DT Template Set                                                 */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_template_setDT (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz)
{
  	ViUInt32 i,retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    

    if (avopt62_invalidViBooleanRange (template))
       	return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViReal64Range (shiftXHz, -8.0e+9, 8.0e+9))
       	return VI_ERROR_PARAMETER3;   
    if (avopt62_invalidViReal64Range (shiftYdB, -200.0, 100))
       	return VI_ERROR_PARAMETER4; 
	if (avopt62_invalidViReal64Range (deltaXHz, -100e+6, 100e+6 ))
       	return VI_ERROR_PARAMETER5; 

    Fmt (wrtBuf, "%s<DTSTMPL %s; DTSTMPLSX %fHZ; DTSTMPLSY %fDB; DTSTMPLDX %fHZ",
    	switchArr[template],
    	shiftXHz, shiftYdB,deltaXHz);
 
    total += NumFmtdBytes();
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;       
}



/*===========================================================================*/
/* Function: DT Template Set                                                 */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_template_editDT (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB)
{
  	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;   

    if (avopt62_invalidViInt16Range (operation, 0, 2))
        return VI_ERROR_PARAMETER2;

	switch ( operation )
	{
	case 0: /* Copy */
		Fmt (wrtBuf, "%s<DTSTMPLCP");	
		break;
	case 1: /* entry */
		if (avopt62_invalidViReal64Range (templateFreqHz, -100.0e+9, 100.0e+9))
        	return VI_ERROR_PARAMETER3;
    	if (avopt62_invalidViReal64Range (templateLeveldB, -200, 100))
        	return VI_ERROR_PARAMETER4;
        Fmt (wrtBuf, "%s<DTSTMPLED %fHZ,%fDB",templateFreqHz,templateLeveldB);
		break;
	case 2: /* Init */
		Fmt (wrtBuf, "%s<DTSTMPLCLR");   
		break;
	}

    total += NumFmtdBytes();	
	
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;       	
	
}

/*===========================================================================*/
/* Function: DT Marker Edit                                                  */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_marker_editDT (ViSession instrumentHandle,
                                        ViInt16 operation,
                                        ViInt16 refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    

	if (avopt62_invalidViInt16Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;

	switch (operation)
	{
		case 0: /*  STD */
			Fmt (wrtBuf, "%s<DTSMKRCP");	
			break;
			
		case 1: /* Entry */
			if (avopt62_invalidViBooleanRange (offsetMkrType))
        		return VI_ERROR_PARAMETER5;
    		if (avopt62_invalidViReal64Range(offsetFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER6;
    		if (avopt62_invalidViReal64Range(bandWidthHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER7;
    		if (avopt62_invalidViReal64Range(limitPowerdB, -200, 100))
        		return VI_ERROR_PARAMETER8;			
			
			
           	Fmt (wrtBuf, "%s<DTSMKRED %d,%fHZ,%fHZ,%fDB",offsetMkrType,offsetFreqHz,bandWidthHz,limitPowerdB);
			
			break;
		case 2:	/* Init */
			Fmt (wrtBuf, "%s<DTSMKRCLR");
			break;
		case 3: /* Ref */
			if (avopt62_invalidViInt16Range (refMarkerType, 0, 2))
        		return VI_ERROR_PARAMETER3;
    		if (avopt62_invalidViReal64Range (refBandWidthHz, 0, 1.0e+9))
        		return VI_ERROR_PARAMETER4;           	

			Fmt (wrtBuf, "%s<DTSMKRCLR; DTSMKRED %d,0HZ,%fHZ,0DB",refMarkerType,refBandWidthHz);

			break;
	}
	total += NumFmtdBytes();
	
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;      
}

/*===========================================================================*/
/* Function: DM Parameter Set                                                */
/* Purpose:  This function sets the parameters for Due to modulation		 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_parameter_setDM (ViSession instrumentHandle,
                                          ViBoolean settoSTD,
                                          ViBoolean freqSetting,
                                          ViInt16 detector, ViInt16 result,
                                          ViBoolean refPower,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViBoolean judgement,
                                          ViReal64 templateLimit,
                                          ViReal64 symbolRate, ViReal64 rolloff)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<DTMSETSTD");	
	   total += NumFmtdBytes();            
	}
	else
	{
   		if (avopt62_invalidViBooleanRange (freqSetting))
        	return VI_ERROR_PARAMETER3;       
    	if (avopt62_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER4;
    	if (avopt62_invalidViInt16Range (result, 0, 2))
        	return VI_ERROR_PARAMETER5;
  		if (avopt62_invalidViBooleanRange (refPower))
        	return VI_ERROR_PARAMETER6;           	
   		if (avopt62_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER7;
   		if (avopt62_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_PARAMETER8; 
   		if (avopt62_invalidViBooleanRange (judgement))
        	return VI_ERROR_INSTR_PARAMETER9;  
   		if (avopt62_invalidViReal64Range (symbolRate, 1.0, 1.0e+9))
        	return VI_ERROR_INSTR_PARAMETER11;  
   		if (avopt62_invalidViReal64Range (rolloff, 0.01, 0.99))
        	return VI_ERROR_INSTR_PARAMETER11;  
  	
        Fmt (wrtBuf, "%s<DTM%s; DTM%s; DTM%s; DTM%s; DTM%s; DTMTMPLPW %s; DTMTMPLBTM %fDB; DTMJDG %s; DTMSYMRT %fHZ; DTMRFACT %f",
        	FreqSetArr[freqSetting], DetArr[detector], dtsResultArr[result],
        	refMkrArr[refPower], LvlUnitArr[displayUnit],switchArr[TMPLCoupleofPWR],
			templateLimit, switchArr[judgement], symbolRate, rolloff);
    	total += NumFmtdBytes();	
	}
        
 
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;	    	    
}

/*===========================================================================*/
/* Function: DM Template Set                                                 */
/* Purpose:  This function sets the parameters for Due to modulation		 */
/*	measurement. This function sets for Template.			                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_template_setDM (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz)
{
  	ViUInt32 i,retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    

    if (avopt62_invalidViBooleanRange (template))
       	return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViReal64Range (shiftXHz, -8.0e+9, 8.0e+9))
       	return VI_ERROR_PARAMETER3;   
    if (avopt62_invalidViReal64Range (shiftYdB, -200.0, 100))
       	return VI_ERROR_PARAMETER4; 
	if (avopt62_invalidViReal64Range (deltaXHz, -100e+6, 100e+6 ))
       	return VI_ERROR_PARAMETER5; 

    Fmt (wrtBuf, "%s<DTMTMPL %s; DTMTMPLSX %fHZ; DTMTMPLSY %fDB; DTMTMPLDX %fHZ",
    	switchArr[template], shiftXHz, shiftYdB, deltaXHz);
 
    total += NumFmtdBytes();
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;           
}

/*===========================================================================*/
/* Function: DM Template Edit                                                */
/* Purpose:  This function edets the parameters for Due to modulation		 */
/*	measurement. This function sets for Template.			                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_template_editDM (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLevel)
{
   	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;   

    if (avopt62_invalidViInt16Range (operation, 0, 2))
        return VI_ERROR_PARAMETER2;

	switch ( operation )
	{
	case 0: /* Copy */
		Fmt (wrtBuf, "%s<DTMTMPLCP");	
		break;
	case 1: /* entry */
		if (avopt62_invalidViReal64Range (templateFreqHz, -100.0e+9, 100.0e+9))
        	return VI_ERROR_PARAMETER3;
        Fmt (wrtBuf, "%s<DTMTMPLED %fHZ,%f[e]",templateFreqHz,templateLevel);
		break;
	case 2: /* Init */
		Fmt (wrtBuf, "%s<DTMTMPLCLR");   
		break;
	}

    total += NumFmtdBytes();	
	
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;   
}

/*===========================================================================*/
/* Function: DM Marker Table Edit  											 */
/* Purpose:  This function sets the parameters for Due to modulation		 */
/*	measurement. This function sets for Template.			                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_marker_editDM (ViSession instrumentHandle,
                                        ViInt16 operation,
                                        ViInt16 refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    

	if (avopt62_invalidViInt16Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;

	switch (operation)
	{
		case 0: /*  STD */
			Fmt (wrtBuf, "%s<DTMMKRCP");	
			break;
			
		case 1: /* Entry */
			if (avopt62_invalidViBooleanRange (offsetMkrType))
        		return VI_ERROR_PARAMETER5;
    		if (avopt62_invalidViReal64Range(offsetFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER6;
    		if (avopt62_invalidViReal64Range(bandWidthHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER7;
    		if (avopt62_invalidViReal64Range(limitPowerdB, -200, 100))
        		return VI_ERROR_PARAMETER8;			
			
			Fmt (wrtBuf, "%s<DTMMKRED %d,%fHZ,%fHZ,%fDB",offsetMkrType,offsetFreqHz,bandWidthHz,limitPowerdB);

			break;
		case 2:	/* Init */
			Fmt (wrtBuf, "%s<DTMMKRCLR");
			break;
		case 3: /* Ref */
			if (avopt62_invalidViInt16Range (refMarkerType, 0, 2))
        		return VI_ERROR_PARAMETER3;
    		if (avopt62_invalidViReal64Range (refBandWidthHz, 0, 1.0e+9))
        		return VI_ERROR_PARAMETER4; 
			Fmt (wrtBuf, "%s<DTMMKRCLR; DTMMKRED %d,0HZ,%fHZ,0DB",refMarkerType,refBandWidthHz);
			break;
	}
	total += NumFmtdBytes();
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;          
}


/*===========================================================================*/
/* Function: InBand Spurious Parameter Set                                   */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/


ViStatus CVIFUNC avopt62_parameter_setINSPR (ViSession instrumentHandle,
                                             ViBoolean settoSTD,
                                             ViBoolean freqSetting,
                                             ViInt16 detector,
                                             ViBoolean refPower,
                                             ViReal64 peakMkrYDelta,
                                             ViInt16 result, ViInt16 displayUnit,
                                             ViBoolean TMPLCoupleofPWR,
                                             ViReal64 templateLimit,
                                             ViBoolean judgement)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<SPRSETSTD");	
	}
	else
	{
   		if (avopt62_invalidViBooleanRange (freqSetting))
        	return VI_ERROR_PARAMETER3;       
    	if (avopt62_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER4;
  		if (avopt62_invalidViBooleanRange (refPower))
        	return VI_ERROR_PARAMETER5; 
    	if (avopt62_invalidViReal64Range (peakMkrYDelta, 0.1, 10.0))
        	return VI_ERROR_PARAMETER6;           	
   		if (avopt62_invalidViInt16Range (result, 0, 2))
        	return VI_ERROR_PARAMETER7;        	
   		if (avopt62_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER8;
   		if (avopt62_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_INSTR_PARAMETER9; 
   		if (avopt62_invalidViBooleanRange (judgement))
        	return VI_ERROR_INSTR_PARAMETER11; 

        Fmt (wrtBuf, "%s<SPR%s; SPR%s;SPRPKMKY %fENT; SPR%s; SPR%s; SPR%s;SPRTMPLPW %s;SPRTMPLBTM %f[e]; SPRJDG %s",
        	FreqSetArr[freqSetting], DetArr[detector],peakMkrYDelta, dtsResultArr[result],
        	refMkrArr[refPower], LvlUnitArr[displayUnit],switchArr[TMPLCoupleofPWR],
			templateLimit, switchArr[judgement]);
	}
        
    total += NumFmtdBytes();
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;	    	        
}

/*===========================================================================*/
/* Function: InBand Spurious Template Set                                    */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/

ViStatus CVIFUNC avopt62_template_setINSPR (ViSession instrumentHandle,
                                            ViBoolean template,
                                            ViReal64 shiftXHz, ViReal64 shiftYdB,
                                            ViReal64 deltaXHz)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE],wrtBuf2[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    


    if (avopt62_invalidViBooleanRange (template))
       	return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViReal64Range (shiftXHz, -8.0e+9, 8.0e+9))
       	return VI_ERROR_PARAMETER3;   
    if (avopt62_invalidViReal64Range (shiftYdB, -200.0, 100))
       	return VI_ERROR_PARAMETER4; 
	if (avopt62_invalidViReal64Range (deltaXHz, -100e+6, 100e+6))
       	return VI_ERROR_PARAMETER5; 

    Fmt (wrtBuf, "%s<SPRTMPL %s; SPRTMPLSX %fHZ; SPRTMPLSY %fDB; SPRTMPLDX %fHZ",switchArr[template],
    	shiftXHz, shiftYdB,deltaXHz);
 
    total += NumFmtdBytes();
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;	        
}

/*===========================================================================*/
/* Function: InBand Spurious Template Edit                                   */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_template_editINSPR (ViSession instrumentHandle,
                                             ViInt16 operation,
                                             ViReal64 templateFreqHz,
                                             ViReal64 templateLevel)
{
   	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;   

    if (avopt62_invalidViInt16Range (operation, 0, 2))
        return VI_ERROR_PARAMETER2;

	switch ( operation )
	{
	case 0: /* Copy */
		Fmt (wrtBuf, "%s<SPRTMPLCP");	
		break;
	case 1: /* entry */
		if (avopt62_invalidViReal64Range (templateFreqHz, -100.0e+9, 100.0e+9))
        	return VI_ERROR_PARAMETER3;
        Fmt (wrtBuf, "%s<SPRTMPLED %fHZ,%f[e]",templateFreqHz,templateLevel);
		break;
	case 2: /* Init */
		Fmt (wrtBuf, "%s<SPRTMPLCLR");   
		break;
	}

    total += NumFmtdBytes();	
	
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;   
}    


/*===========================================================================*/
/* Function: InBand Spurious Marker Set                                      */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_marker_editINSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean refMarkerType,
                                           ViReal64 refBandWidthHz,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz,
                                           ViReal64 limitPower)
{
 	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;    

	if (avopt62_invalidViInt16Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;

	switch (operation)
	{
		case 0: /*  STD */
			Fmt (wrtBuf, "%s<SPRMKRCP");	
			break;
			
		case 1: /* Entry */
    			if (avopt62_invalidViReal64Range(startFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER5;
    			if (avopt62_invalidViReal64Range(stopFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER6;

            	Fmt (wrtBuf, "%s<SPRMKRED 0,%fHZ,%fHZ,%f[e]", startFreqHz, stopFreqHz, limitPower);
			break;
		case 2:	/* Init */
			Fmt (wrtBuf, "%s<SPRMKRCLR");
			break;
		case 3: /* Ref */
			if (avopt62_invalidViBooleanRange (refMarkerType))
        		return VI_ERROR_PARAMETER3;
    		if (avopt62_invalidViReal64Range (refBandWidthHz, 0, 1.0e+9))
        		return VI_ERROR_PARAMETER4;     
			Fmt (wrtBuf, "%s<SPRMKRCLR; SPRMKRED %d,0HZ,%fHZ,0DB",refMarkerType,refBandWidthHz);
			break;
	}
	
	total += NumFmtdBytes();
	
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: OutBand Spurious Parameter Set                                  */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_parameter_setOUTSPR (ViSession instrumentHandle,
                                              ViBoolean settoSTD,
                                              ViInt16 detector,
                                              ViReal64 peakMkrYDelta,
                                              ViInt16 displayUnit,
                                              ViBoolean judgement,
                                              ViBoolean preselector)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (avopt62_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<FDSSETSTD");	
	}
	else
	{
	
    	if (avopt62_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt62_invalidViReal64Range (peakMkrYDelta, 0.1, 10.0))
        	return VI_ERROR_PARAMETER4;           	
   		if (avopt62_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER5;
   		if (avopt62_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER6; 
   		if (avopt62_invalidViBooleanRange (preselector))
        	return VI_ERROR_PARAMETER7; 

        Fmt (wrtBuf, "%s<FDS%s;FDSPKMKY %fENT; FDS%s; FDSJDG %s; FDS%s",
        	DetArr[detector], peakMkrYDelta,
        	LvlUnitArr[displayUnit], switchArr[judgement],preselArr[preselector]);
	}
        
    total += NumFmtdBytes();
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: OutBand Spurious Table Set                                      */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/

ViStatus CVIFUNC avopt62_table_editOUTSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViInt16 tableNumber,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz, ViReal64 RBW,
                                           ViReal64 VBW, ViReal64 sweepTime,
                                           ViReal64 limitLevel)
{
 	ViUInt32 total = 0;       
    ViUInt32 i, retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

	
	if (avopt62_invalidViInt16Range (operation, 0, 4))
        return VI_ERROR_PARAMETER2;
	if (avopt62_invalidViInt16Range (tableNumber, 1, 3))
        return VI_ERROR_PARAMETER3;

    switch (operation)
    {
    	case 0: /* Load */
    		Fmt (wrtBuf, "%s<FDSTBL %d; FDSLD",tableNumber);
    		break;
    	case 1: /* Save */
    		Fmt (wrtBuf, "%s<FDSTBL %d; FDSSV",tableNumber);
    		break;
    	case 2: /* Sel */
    		Fmt (wrtBuf, "%s<FDSTBL %d",tableNumber);
    		break;
    	case 3: /* Edit */
    		if (avopt62_invalidViReal64Range (startFreqHz, -200e+6, 8e+9))
        		return VI_ERROR_PARAMETER4;
			if (avopt62_invalidViReal64Range (stopFreqHz, 0.0, 8.2e+9))
        		return VI_ERROR_PARAMETER5;
			if (avopt62_invalidViReal64Range (RBW, 10.0, 10e+6))
        		return VI_ERROR_PARAMETER6;
        	if (avopt62_invalidViReal64Range (VBW, 1.0, 10e+6))
        		return VI_ERROR_PARAMETER7;
        	if (avopt62_invalidViReal64Range (sweepTime, 1e-6, 1000))
        		return VI_ERROR_PARAMETER8;        	

     		Fmt (wrtBuf, "%s<FDSTBLED %fHZ,%fHZ, %fHZ, %fHZ, %f[e]SC,%f[e]",
     			startFreqHz,stopFreqHz,RBW,VBW,sweepTime,limitLevel);
    		break;
    	case 4: /* Init */
    		Fmt (wrtBuf, "%s<FDSCLR");
    	
    		break;
    }
   total += NumFmtdBytes();
	
	if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Set 3GPP measurement parameters in down link and slot mode.	 */
/* Purpose:  This function sets the 3GPP measurement parameters.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_parameter_setDownSlot3GPP (ViSession instrumentHandle,
                                                     ViBoolean scrmblCodeDef,
                                                     ViInt32 scrmblCodeNo, ViBoolean trgMode,
                                                     ViBoolean extTrgSlp, ViReal64 extTrgDly,
                                                     ViBoolean srchMode,
                                                     ViInt16 primCPICHSF, ViInt16 primCPICHNo,
                                                     ViInt16 actCHDetect, ViInt16 analysisRate,
                                                     ViInt16 measUnit, ViInt16 msStrPos,
                                                     ViInt16 threshold, ViBoolean phaseInverse,
                                                     ViBoolean freqErr, ViBoolean transmitTm)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (avopt62_invalidViBooleanRange (scrmblCodeDef))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViInt32Range (scrmblCodeNo, 0, 262142))
       	return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViBooleanRange (trgMode))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViBooleanRange (extTrgSlp))
        return VI_ERROR_PARAMETER5;
    if (avopt62_invalidViReal64Range (extTrgDly, -5120.0, 5120.0))
       	return VI_ERROR_PARAMETER6;
    if (avopt62_invalidViBooleanRange (srchMode))
        return VI_ERROR_PARAMETER7;
    if (avopt62_invalidViInt16Range (primCPICHSF, 4, 512))
       	return VI_ERROR_PARAMETER8;
    if (avopt62_invalidViInt16Range (primCPICHNo, 0, 511))
       	return VI_ERROR_INSTR_PARAMETER9;
    if (avopt62_invalidViInt16Range (actCHDetect, 0, 7))
        return VI_ERROR_INSTR_PARAMETER10;
    if (avopt62_invalidViInt16Range (analysisRate, 0, 8))
       	return VI_ERROR_INSTR_PARAMETER11;
    if (avopt62_invalidViInt16Range (measUnit, 1, 640))
       	return VI_ERROR_INSTR_PARAMETER12;
    if (avopt62_invalidViInt16Range (msStrPos, 0, 140))
       	return VI_ERROR_INSTR_PARAMETER13;
    if (avopt62_invalidViInt16Range (threshold, -40, -5))
       	return VI_ERROR_INSTR_PARAMETER14;
    if (avopt62_invalidViBooleanRange (phaseInverse))
        return VI_ERROR_INSTR_PARAMETER15;
    if (avopt62_invalidViBooleanRange (freqErr))
        return VI_ERROR_INSTR_PARAMETER16;
    if (avopt62_invalidViBooleanRange (transmitTm))
        return VI_ERROR_INSTR_PARAMETER17;
        
    Fmt (wrtBuf, "%s<%s; 3GPDNSCNO %d; 3GPDN%s; 3GPDN%s; 3GPDNTRGDLY %f; %s; 3GPCPICHSF %d; 3GPCPICHNO %d; %s; %s; 3GPDNMUNIT %d; 3GPDNMSTSLT %d; 3GPDNTHRSH %dDB; 3GPDN%s; 3GPDN%s; 3GPTRNSTM %s",
		codeDefArr[scrmblCodeDef], scrmblCodeNo, trgModeArr[trgMode], trgSlopeArr[extTrgSlp],
		extTrgDly, srchModeArr[srchMode], primCPICHSF, primCPICHNo, actCHDtctArr[actCHDetect],
		downAnaRateArr[analysisRate], measUnit, msStrPos, threshold, phsInvArr[phaseInverse],
		freqErrArr[freqErr], switchArr[transmitTm]);

    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Set 3GPP measurement parameters in down link and frame mode.	 */
/* Purpose:  This function sets the 3GPP measurement parameters.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_parameter_setDownFrame3GPP (ViSession instrumentHandle,
                                                      ViBoolean scrmblCodeDef,
                                                      ViInt32 scrmblCodeNo, ViBoolean trgMode,
                                                      ViBoolean extTrgSlp, ViReal64 extTrgDly,
                                                      ViBoolean srchMode,
                                                      ViInt16 primCPICHSF, ViInt16 primCPICHNo,
                                                      ViInt16 actCHDetect,
                                                      ViInt16 msChnlSF, ViInt16 msChnlNo,
                                                      ViInt16 msSlots, ViInt16 msStrPos,
                                                      ViInt16 threshold, ViBoolean phaseInverse)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (avopt62_invalidViBooleanRange (scrmblCodeDef))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViInt32Range (scrmblCodeNo, 0, 262142))
       	return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViBooleanRange (trgMode))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViBooleanRange (extTrgSlp))
        return VI_ERROR_PARAMETER5;
    if (avopt62_invalidViReal64Range (extTrgDly, -5120.0, 5120.0))
       	return VI_ERROR_PARAMETER6;
    if (avopt62_invalidViBooleanRange (srchMode))
        return VI_ERROR_PARAMETER7;
    if (avopt62_invalidViInt16Range (primCPICHSF, 4, 512))
       	return VI_ERROR_PARAMETER8;
    if (avopt62_invalidViInt16Range (primCPICHNo, 0, 511))
       	return VI_ERROR_INSTR_PARAMETER9;
    if (avopt62_invalidViInt16Range (actCHDetect, 0, 7))
        return VI_ERROR_INSTR_PARAMETER10;
    if (avopt62_invalidViInt16Range (msChnlSF, 4, 512))
       	return VI_ERROR_INSTR_PARAMETER11;
    if (avopt62_invalidViInt16Range (msChnlNo, 0, 511))
       	return VI_ERROR_INSTR_PARAMETER12;
    if (avopt62_invalidViInt16Range (msSlots, 1, 30))
       	return VI_ERROR_INSTR_PARAMETER13;
    if (avopt62_invalidViInt16Range (msStrPos, 0, 140))
       	return VI_ERROR_INSTR_PARAMETER14;
    if (avopt62_invalidViInt16Range (threshold, -40, -5))
       	return VI_ERROR_INSTR_PARAMETER15;
    if (avopt62_invalidViBooleanRange (phaseInverse))
        return VI_ERROR_INSTR_PARAMETER16;

    Fmt (wrtBuf, "%s<%s; 3GPDNSCNO %d; 3GPDN%s; 3GPDN%s; 3GPDNTRGDLY %f; %s; 3GPCPICHSF %d; 3GPCPICHNO %d; %s; 3GPDNMCHSF %d; 3GPDNMCHNO %d; 3GPDNMSLOT %d; 3GPDNMSTFRM %d; 3GPDNTHRSH %dDB; 3GPDN%s",
		codeDefArr[scrmblCodeDef], scrmblCodeNo, trgModeArr[trgMode], trgSlopeArr[extTrgSlp],
		extTrgDly, srchModeArr[srchMode], primCPICHSF, primCPICHNo, actCHDtctArr[actCHDetect],
		msChnlSF, msChnlNo, msSlots, msStrPos, threshold, phsInvArr[phaseInverse]);

    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Set 3GPP measurement parameters in up link and slot mode.		 */
/* Purpose:  This function sets the 3GPP measurement parameters.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_parameter_setUpSlot3GPP (ViSession instrumentHandle,
                                                   ViInt32 scrmblCodeNo, ViBoolean trgMode,
                                                   ViBoolean extTrgSlp, ViReal64 extTrgDly,
                                                   ViInt16 DPCCHSF, ViInt16 DPCCHNo,
                                                   ViInt16 analysisRate, ViInt16 measUnit,
                                                   ViInt16 msStrPos,ViInt16 threshold,
                                                   ViBoolean phaseInverse, ViBoolean freqErr)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (avopt62_invalidViInt32Range (scrmblCodeNo, 0, 16777215))
       	return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViBooleanRange (trgMode))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViBooleanRange (extTrgSlp))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViReal64Range (extTrgDly, -5120.0, 5120.0))
       	return VI_ERROR_PARAMETER5;
    if (avopt62_invalidViInt16Range (DPCCHSF, 4, 256))
       	return VI_ERROR_PARAMETER6;
    if (avopt62_invalidViInt16Range (DPCCHNo, 0, 255))
       	return VI_ERROR_PARAMETER7;
    if (avopt62_invalidViInt16Range (analysisRate, 0, 6))
       	return VI_ERROR_PARAMETER8;
    if (avopt62_invalidViInt16Range (measUnit, 1, 640))
       	return VI_ERROR_INSTR_PARAMETER9;
    if (avopt62_invalidViInt16Range (msStrPos, 0, 140))
       	return VI_ERROR_INSTR_PARAMETER10;
    if (avopt62_invalidViInt16Range (threshold, -40, -5))
       	return VI_ERROR_INSTR_PARAMETER11;
    if (avopt62_invalidViBooleanRange (phaseInverse))
        return VI_ERROR_INSTR_PARAMETER12;
    if (avopt62_invalidViBooleanRange (freqErr))
        return VI_ERROR_INSTR_PARAMETER13;
    
    Fmt (wrtBuf, "%s<3GPUPSCNO %d; 3GPUP%s; 3GPUP%s; 3GPUPTRGDLY %f; 3GPDPCCHSF %d; 3GPDPCCHNO %d; %s; 3GPUPMUNIT %d; 3GPUPMSTSLT %d; 3GPUPTHRSH %dDB; 3GPUP%s; 3GPUP%s",
		scrmblCodeNo, trgModeArr[trgMode], trgSlopeArr[extTrgSlp], extTrgDly, 
		DPCCHSF, DPCCHNo, upAnaRateArr[analysisRate], measUnit, msStrPos, 
		threshold, phsInvArr[phaseInverse], freqErrArr[freqErr]);

    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Set 3GPP measurement parameters in up link and frame mode.		 */
/* Purpose:  This function sets the 3GPP measurement parameters.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_parameter_setUpFrame3GPP (ViSession instrumentHandle,
                                                    ViInt32 scrmblCodeNo, ViBoolean trgMode,
                                                    ViBoolean extTrgSlp, ViReal64 extTrgDly,
                                                    ViInt16 DPCCHSF, ViInt16 DPCCHNo,
                                                    ViInt16 msChnlSF, ViInt16 msChnlNo,
                                                    ViBoolean msBranch, ViInt16 msSlots,
                                                    ViInt16 msStrPos, ViInt16 threshold,
                                                    ViBoolean phaseInverse)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (avopt62_invalidViInt32Range (scrmblCodeNo, 0, 16777215))
       	return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViBooleanRange (trgMode))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViBooleanRange (extTrgSlp))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViReal64Range (extTrgDly, -5120.0, 5120.0))
       	return VI_ERROR_PARAMETER5;
    if (avopt62_invalidViInt16Range (DPCCHSF, 4, 256))
       	return VI_ERROR_PARAMETER6;
    if (avopt62_invalidViInt16Range (DPCCHNo, 0, 255))
       	return VI_ERROR_PARAMETER7;
    if (avopt62_invalidViInt16Range (msChnlSF, 4, 256))
       	return VI_ERROR_PARAMETER8;
    if (avopt62_invalidViInt16Range (msChnlNo, 0, 255))
       	return VI_ERROR_INSTR_PARAMETER9;
    if (avopt62_invalidViBooleanRange (msBranch))
        return VI_ERROR_INSTR_PARAMETER10;
    if (avopt62_invalidViInt16Range (msSlots, 1, 30))
       	return VI_ERROR_INSTR_PARAMETER11;
    if (avopt62_invalidViInt16Range (msStrPos, 0, 140))
       	return VI_ERROR_INSTR_PARAMETER12;
    if (avopt62_invalidViInt16Range (threshold, -40, -5))
       	return VI_ERROR_INSTR_PARAMETER13;
    if (avopt62_invalidViBooleanRange (phaseInverse))
        return VI_ERROR_INSTR_PARAMETER14;
    
    Fmt (wrtBuf, "%s<3GPUPSCNO %d; 3GPUP%s; 3GPUP%s; 3GPUPTRGDLY %f; 3GPDPCCHSF %d; 3GPDPCCHNO %d; 3GPUPMCHSF %d; 3GPUPMCHNO %d; %s; 3GPUPMSLOT %d; 3GPUPMSTFRM %d; 3GPUPTHRSH %dDB; 3GPUP%s",
		scrmblCodeNo, trgModeArr[trgMode], trgSlopeArr[extTrgSlp], extTrgDly, 
		DPCCHSF, DPCCHNo, msChnlSF, msChnlNo, msBranchArr[msBranch], msSlots, 
		msStrPos, threshold, phsInvArr[phaseInverse]);

    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Set User Table's Size                                           */
/* Purpose:  This function sets the number of channels which are multiplied  */
/*			 to the measurement signal.										 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_tableSize_set3GPP (ViSession instrumentHandle,
                                             ViInt16 mltChnlNo)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViInt16Range (mltChnlNo, 1, 32))
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuf, "%s<3GPMLTNUM %d", mltChnlNo);
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Edit User Table                                                 */
/* Purpose:  This function edits the 3GPP user table.						 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_table_edit3GPP (ViSession instrumentHandle,
                                          ViInt16 editChnlNo, ViInt16 sf,
                                          ViInt16 number)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViInt16Range (editChnlNo, 1, 31))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViInt16Range (sf, 4, 512))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViInt16Range (number, 0, 511))
        return VI_ERROR_PARAMETER4;
    
    Fmt (wrtBuf, "%s<3GPCH%dSF %d; 3GPCH%dNUM %d", editChnlNo, sf, editChnlNo, number);
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Set QPSK measurement parameters.								 */
/* Purpose:  This function sets the QPSK measurement parameters.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_parameter_setQPSK (ViSession instrumentHandle,
                                             ViBoolean rtNyq, ViInt16 msRange,
                                             ViBoolean trgMode,
                                             ViBoolean extTrgSlp, ViReal64 extTrgDly,
                                             ViBoolean judge, ViReal64 lmtRho)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViBooleanRange (rtNyq))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViInt16Range (msRange, 4, 20))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViBooleanRange (trgMode))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViBooleanRange (extTrgSlp))
        return VI_ERROR_PARAMETER5;
    if (avopt62_invalidViReal64Range (extTrgDly, -5120.0, 5120.0))
        return VI_ERROR_PARAMETER6;
    if (avopt62_invalidViBooleanRange (judge))
        return VI_ERROR_PARAMETER7;
    if (avopt62_invalidViReal64Range (lmtRho, 0.0001, 1.0))
        return VI_ERROR_PARAMETER8;
        
    Fmt (wrtBuf, "%s<QPRNYQ %s; QPMRNG %d; QP%s; QP%s; QPTRGDLY %f; QPLMJDG %s; QPLMRHO %f",
    	switchArr[rtNyq], msRange, trgModeArr[trgMode], trgSlopeArr[extTrgSlp],
    	extTrgDly, switchArr[judge], lmtRho);
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: TX Power                                                        */
/* Purpose:  This function sets the parameters for TX Power measurement.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_parameter_setTxPower (ViSession instrumentHandle,
                                                ViBoolean rtNyq, ViBoolean trgMode,
                                                ViBoolean extTrgSlp, ViReal64 extTrgDly)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViBooleanRange (rtNyq))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViBooleanRange (trgMode))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViBooleanRange (extTrgSlp))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViReal64Range (extTrgDly, -5120.0, 5120.0))
        return VI_ERROR_PARAMETER5;
        
    Fmt (wrtBuf, "%s<TXRNYQ %s; TX%s; TX%s; TXTRGDLY %f", switchArr[rtNyq],
    	trgModeArr[trgMode], trgSlopeArr[extTrgSlp], extTrgDly);
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Set power vs time measurement graph's scale.					 */
/* Purpose:  This function sets the scales for the graph of the power vs 	 */
/*			 time measurement.												 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_scale_setPwrTime (ViSession instrumentHandle,
                                            ViInt16 ySclUp, ViInt16 ySclRng,
                                            ViBoolean pwrUnit)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViInt16Range (ySclUp, -20, 70))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViInt16Range (ySclRng, 10, 50))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViBooleanRange (pwrUnit))
        return VI_ERROR_PARAMETER4;
        
    Fmt (wrtBuf, "3GPPTYUPR %d; 3GPPTYRNG %d; 3GPPT%s", ySclUp, ySclRng, pwrUnitArr[pwrUnit]);
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Set power vs time measurement parameters.						 */
/* Purpose:  This function sets the parameters of the power vs time 		 */
/*			 measurement.													 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_parameter_setPwrTime (ViSession instrumentHandle,
                                                ViBoolean rtNyq, ViInt16 trgMode,
                                                ViBoolean trgSlp, ViInt16 trgLvl,
                                                ViReal64 trgDly, ViInt16 msLength,
                                                ViBoolean grPltTyp, ViBoolean omitTr)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViBooleanRange (rtNyq))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViInt16Range (trgMode, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViBooleanRange (trgSlp))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViInt16Range (trgLvl, 0, 100))
        return VI_ERROR_PARAMETER5;
    if (avopt62_invalidViReal64Range (trgDly, -40.0e-3, 666.66e-6))
        return VI_ERROR_PARAMETER6;
    if (avopt62_invalidViInt16Range (msLength, 2, 62))
        return VI_ERROR_PARAMETER7;
    if (avopt62_invalidViBooleanRange (grPltTyp))
        return VI_ERROR_PARAMETER8;
    if (avopt62_invalidViBooleanRange (omitTr))
        return VI_ERROR_INSTR_PARAMETER9;
        
    Fmt (wrtBuf, "%s<3GPPTRNYQ %s; %s; 3GPPT%s; 3GPPTTRGLVL %d, 3GPPTTRGDLY %f[e], 3GPPTMLEN %d; %s; 3GPPTOMIT %s",
    	switchArr[rtNyq], pwrtmTrgModeArr[trgMode], trgSlopeArr[trgSlp], trgLvl, trgDly,
    	msLength, grPltTypArr[grPltTyp], switchArr[omitTr]);
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Set CCDF measurement graph's scale.							 */
/* Purpose:  This function sets the scales for the graph of the CCDF	 	 */
/*			 measurement.													 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_scale_setCCDF (ViSession instrumentHandle,
                                         ViInt16 xSclMax, ViInt16 xSclRng,
                                         ViBoolean pwrUnit)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViInt16Range (xSclMax, -20, 70))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViInt16Range (xSclRng, 10, 50))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViBooleanRange (pwrUnit))
        return VI_ERROR_PARAMETER4;
        
    Fmt (wrtBuf, "3GPCCDFXMAX %d; 3GPCCDFXRNG %d; 3GPCCDF%s", xSclMax, xSclRng, pwrUnitArr[pwrUnit]);
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: CCDF measurement parameters.									 */
/* Purpose:  This function sets the parameters of the CCDF measurement.		 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_parameter_setCCDF (ViSession instrumentHandle,
                                             ViBoolean rtNyq, ViBoolean trgMode,
                                             ViBoolean trgSlp, ViReal64 trgDly,
                                             ViInt32 msLength)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViBooleanRange (rtNyq))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViBooleanRange (trgMode))
        return VI_ERROR_PARAMETER3;
    if (avopt62_invalidViBooleanRange (trgSlp))
        return VI_ERROR_PARAMETER4;
    if (avopt62_invalidViReal64Range (trgDly, -250.0e-6, 250.0e-6))
        return VI_ERROR_PARAMETER5;
    if (avopt62_invalidViInt32Range (msLength, 10000, 100000000))
        return VI_ERROR_PARAMETER6;
    
    Fmt (wrtBuf, "%s<3GPCCDFRNYQ %s; 3GPCCDF%s; 3GPCCDF%s; 3GPCCDFTRGDLY %f[e]; 3GPCCDFMLEN %d",
    	switchArr[rtNyq], trgModeArr[trgMode], trgSlopeArr[trgSlp], trgDly, msLength);
    	
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Averaging                                                       */
/* Purpose:  This function sets the averaging times of Burst Envelope		 */
/*           Spectrum, Gated Output Power, On/Off Ratio, OBW,  Channel Power,*/
/*           In Band Spurious, Waveform Quality, Code Domain Power, TX Power */
/*           functions.														 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_average_set (ViSession instrSession, ViInt16 function,
				ViInt16 AVGTime)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViInt16Range (function, 0, 11))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViInt16Range (AVGTime, 1, 999))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<%s %d[b2]", avgFunctionArr[function], AVGTime);

    if ((avopt62_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;
}


/*===========================================================================*/
/* Function: Auto Level Set                                                  */
/* Purpose:  This function is used for Auto Level set in selected meas mode. */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_execute_auto_lvl (ViSession instrumentHandle,
                                           ViInt16 function)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViInt16Range (function, 0, 9))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", autoLvlFunctionArr[function]);

    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Select 3GPP's Graph Type                                        */
/* Purpose:  This function is used for 3GPP's graph type selection.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_selectGraphType3GPP (ViSession instrumentHandle,
                                               ViInt16 grphSlct, ViInt16 dispStr)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViInt16Range (grphSlct, 0, 11))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViInt16Range (dispStr, 0, 2304))
        return VI_ERROR_PARAMETER3;
        
    if (grphSlct==11)
    	Fmt (wrtBuf, "%s<3GPDSPST %d", dispStr);
    	
    else
    	Fmt (wrtBuf, "%s<%s", grphTyp3GPPArr[grphSlct]);
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;
}

/*===========================================================================*/
/* Function: Select QPSK's Graph Type                                        */
/* Purpose:  This function is used for QPSK's graph type selection.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_selectGraphTypeQPSK (ViSession instrumentHandle,
                                               ViInt16 grphSlct, ViInt16 dispStr)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViInt16Range (grphSlct, 0, 10))
        return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViInt16Range (dispStr, 0, 1024))
        return VI_ERROR_PARAMETER3;
        
    if (grphSlct==10)
    	Fmt (wrtBuf, "%s<QPDSPST %d", dispStr);
    	
    else
    	Fmt (wrtBuf, "%s<%s", grphTypQPSKArr[grphSlct]);
    
    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;
}

/*===========================================================================*/
/* Function: Graph Marker                                                    */
/* Purpose:  This function is used to switch graph marker On/Off.			 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_actGraphMkr (ViSession instrSession, ViBoolean marker)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViBooleanRange (marker))
        return VI_ERROR_PARAMETER2;

	if (marker)
	{
	 Fmt (wrtBuf, "%s<MK");   	
	}
	else
	{
	 Fmt (wrtBuf, "%s<MKOFF");
	}

    if ((avopt62_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;
}

/*===========================================================================*/
/* Function: DC Cal                                			                 */
/* Purpose:  This control executes DC component calibration of the 			 */
/*			 instrument.													 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_act_DCCal (ViSession instrumentHandle)
{
    ViUInt32 retCnt = 0;
    ViStatus avopt62_status = VI_SUCCESS;
    
    if ((avopt62_status = viWrite (instrumentHandle, "CLDC", 4, &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Start Measurement                                               */
/* Purpose:  This function starts desired measurement mode of TRANSIENT		 */
/*           section.														 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_actMeasStart (ViSession instrumentHandle,
                                        ViInt16 function)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if (avopt62_invalidViInt16Range (function, 0, 14))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", funcMeasureArr[function]);

    if ((avopt62_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt62_status;

    return avopt62_status;
}

/*===========================================================================*/
/* Function: Check meas mode		                                         */
/* Purpose:  This function returns the mode of instrument.(CW/Transient)     */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_check_measMode (ViSession instrumentHandle,
                                         ViInt16 *returnmode)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    

    if ((avopt62_status = viWrite (instrumentHandle, "SETFUNC?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;

   	if (Scan (rdBuf, "%s>%d[b2]", returnmode) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt62_status;	       
    
}

/*===========================================================================*/
/* Function: Time Domain Power Result                                        */
/* Purpose:  This function returns the result of power measurement in        */
/*           the time domain.												 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_data_TDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "TDPMEAS?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;

   	if (Scan (rdBuf, "%s>%f,%d[b2]", powerLeveldBm,judgement) < 2)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt62_status;	    
}

/*===========================================================================*/
/* Function: On/Off Ratio                                                    */
/* Purpose:  This function returns the result of On/Off ratio measurement of */
/*           the burst signal in the time domain.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_dataOnOffRatio (ViSession instrumentHandle,
                                          ViReal64 *ONLevel, ViReal64 *OFFLevel,
                                          ViReal64 *ONOFFRaito, ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "OORMEAS?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f,%d[b2]", ONLevel, OFFLevel, ONOFFRaito, judgement) < 4)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt62_status;    
}

/*===========================================================================*/
/* Function: TD Spurious Data                                                */
/* Purpose:  This function returns the result of Time Domain Spurious		 */
/*           measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_data_spuriTD (ViSession instrumentHandle,
                                       ViInt16 *tableSize,
                                       ViReal64 frequencyHz[], ViReal64 level[],
                                       ViInt16 judgement[])
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "TDSMEAS?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;

   	if (Scan (rdBuf, "%s>%d[b2]", tableSize) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	for (i=0;i<*tableSize;i++) {
	    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt62_status;
	   	if (Scan (rdBuf, "%s>%f,%f,%d[b2]", &frequencyHz[i], &level[i], &judgement[i]) < 3)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    return avopt62_status;
}

/*===========================================================================*/
/* Function: F Domain Power Result                                        */
/* Purpose:  This function returns the result of power measurement in        */
/*           the time domain.												 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_data_FDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "FDPMEAS?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;

   	if (Scan (rdBuf, "%s>%f,%d[b2]", powerLeveldBm,judgement) < 2)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt62_status;	        
}

/*===========================================================================*/
/* Function: OBW Data                                                        */
/* Purpose:  This function returns the result of occupied bandwidth			 */
/*           measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_dataOBW (ViSession instrSession, ViPReal64 OBW,
               			 		  ViPReal64 lowFreq, ViPReal64 hiFreq,
               			 		  ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrSession, "OBWMEAS?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f,%d[b2]", OBW, lowFreq, hiFreq, judgement) < 4)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt62_status;
}

/*===========================================================================*/
/* Function: Due to Transient Data                                           */
/* Purpose:  This function returns the result of Due to Transient			 */
/*			 measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_data_duetoTRN (ViSession instrumentHandle,
                                        ViInt16 *numberofACP,
                                        ViReal64 ACPDatadBc[],
                                        ViInt16 judgement[])
{
    ViUInt32 i,retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "DTSMEAS?", 8, &retCnt)) < 0)
        return avopt62_status;

    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofACP) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *numberofACP; i++)
	{
	    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt62_status;
	   	if (Scan (rdBuf, "%s>%f,%d[b2]", &ACPDatadBc[i], &judgement[i]) < 2)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;		
	}
	
    return avopt62_status;    
}

/*===========================================================================*/
/* Function: Due to Modulation Data                                          */
/* Purpose:  This function returns the result of Due to Modulation			 */
/*			 measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_data_duetoMOD (ViSession instrumentHandle,
                                        ViInt16 *numberofACP, ViReal64 ACPdBc[],
                                        ViInt16 ACPJudge[])
{
    ViUInt32 i,retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "DTMMEAS?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofACP) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *numberofACP; i++)
	{
	    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt62_status;
	   	if (Scan (rdBuf, "%s>%f,%d[b2]", &ACPdBc[i], &ACPJudge[i]) < 2)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;		
	}
    return avopt62_status;    	    
}

/*===========================================================================*/
/* Function: In Band Spurious                                                */
/* Purpose:  This function returns the result of spurious measurement. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_dataSpurMeas (ViSession instrumentHandle,
                                       ViInt16 *numberofPoint,
                                       ViReal64 frequencyHz[],
                                       ViReal64 level[], ViInt16 judge[])
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "SPRMEAS?", 8, &retCnt)) < 0)
        return avopt62_status;
        
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofPoint) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	for (i=0;i<*numberofPoint;i++) {
	    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt62_status;
	   	
	   	if (Scan (rdBuf, "%s>%f,%f,%d[b2]", &frequencyHz[i], &level[i], &judge[i]) < 3)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    return avopt62_status;
}

/*===========================================================================*/
/* Function: Out Band Spurious                                               */
/* Purpose:  This function returns the result of spurious measurement. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_data_spuriOB (ViSession instrumentHandle,
                                       ViInt16 *numberofPoint,
                                       ViReal64 frequencyHz[],
                                       ViReal64 level[], ViInt16 judgement[])
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "FDSMEAS?", 8, &retCnt)) < 0)
        return avopt62_status;
        
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofPoint) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	for (i=0;i<*numberofPoint;i++) {
	    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt62_status;
	   	
	   	if (Scan (rdBuf, "%s>%f,%f,%d[b2]", &frequencyHz[i], &level[i], &judgement[i]) < 3)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    return avopt62_status;		
}

/*===========================================================================*/
/* Function: 3GPP in down link and slot mode                                 */
/* Purpose:  This function returns the result of 3GPP measurement.	 		 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_dataDownSlot3GPP (ViSession instrumentHandle,
                                            ViReal64 *rho, ViReal64 *tausec,
                                            ViReal64 *tauchip, ViReal64 *crrFrqErr,
                                            ViReal64 *iqOrgOffset, ViReal64 *magErr,
                                            ViReal64 *phsErr, ViReal64 *mdlAcc,
                                            ViReal64 *pkMagErr, ViReal64 *pkPhsErr,
                                            ViReal64 *pkMdlAcc, ViInt16 *slot,
                                            ViInt16 *scrmblCdNo, ViInt16 *scrmblCdGrpNo,
                                            ViReal64 *schPwr, ViReal64 *pwrRatio,
                                            ViReal64 *pkCDE)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPRHO?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", rho) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPTAU?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", tausec) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPTAUCHIP?", 11, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", tauchip) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPCFER?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", crrFrqErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPIQOFS?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", iqOrgOffset) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPMAG?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", magErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPHSE?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", phsErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPMOD?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", mdlAcc) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPKMAG?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pkMagErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPKPHSE?", 10, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pkPhsErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPKMOD?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pkMdlAcc) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPSLOT?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%d[b2]", slot) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPSCCD?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%d[b2]", scrmblCdNo) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPSCGRP?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%d[b2]", scrmblCdGrpNo) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPSCHPWR?", 10, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", schPwr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPRATIO?", 10, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pwrRatio) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPKCDE?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pkCDE) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt62_status;
}

/*===========================================================================*/
/* Function: 3GPP in down link and frame mode                                */
/* Purpose:  This function returns the result of 3GPP measurement.	 		 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_dataDownFrame3GPP (ViSession instrumentHandle,
                                             ViReal64 *tausec, ViReal64 *tauchip,
                                             ViReal64 *crrFrqErr, ViReal64 *iqOrgOffset,
                                             ViReal64 *magErr, ViReal64 *phsErr,
                                             ViReal64 *mdlAcc, ViReal64 *pkMagErr,
                                             ViReal64 *pkPhsErr, ViReal64 *pkMdlAcc,
                                             ViInt16 *slot, ViInt16 *scrmblCdNo,
                                             ViInt16 *scrmblCdGrpNo, ViReal64 *pwrRatio)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPTAU?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", tausec) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPTAUCHIP?", 11, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", tauchip) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPCFER?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", crrFrqErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPIQOFS?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", iqOrgOffset) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPMAG?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", magErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPHSE?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", phsErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPMOD?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", mdlAcc) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPKMAG?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pkMagErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPKPHSE?", 10, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pkPhsErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPKMOD?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pkMdlAcc) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPSLOT?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%d[b2]", slot) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPSCCD?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%d[b2]", scrmblCdNo) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPSCGRP?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%d[b2]", scrmblCdGrpNo) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPRATIO?", 10, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pwrRatio) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt62_status;
}

/*===========================================================================*/
/* Function: 3GPP in up link and slot mode	                                 */
/* Purpose:  This function returns the result of 3GPP measurement.	 		 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_dataUpSlot3GPP (ViSession instrumentHandle,
                                          ViReal64 *rho, ViReal64 *tausec,
                                          ViReal64 *tauchip, ViReal64 *crrFrqErr,
                                          ViReal64 *iqOrgOffset, ViReal64 *magErr,
                                          ViReal64 *phsErr, ViReal64 *mdlAcc,
                                          ViReal64 *pkMagErr, ViReal64 *pkPhsErr,
                                          ViReal64 *pkMdlAcc, ViInt16 *slot,
                                          ViReal64 *pkCDE)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPRHO?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", rho) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPTAU?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", tausec) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPTAUCHIP?", 11, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", tauchip) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPCFER?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", crrFrqErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPIQOFS?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", iqOrgOffset) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPMAG?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", magErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPHSE?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", phsErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPMOD?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", mdlAcc) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPKMAG?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pkMagErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPKPHSE?", 10, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pkPhsErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPKMOD?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pkMdlAcc) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPSLOT?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%d[b2]", slot) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPKCDE?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pkCDE) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt62_status;
}

/*===========================================================================*/
/* Function: 3GPP in up link and frame mode	                                 */
/* Purpose:  This function returns the result of 3GPP measurement.	 		 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_dataUpFrame3GPP (ViSession instrumentHandle,
                                           ViReal64 *tausec, ViReal64 *tauchip,
                                           ViReal64 *crrFrqErr, ViReal64 *iqOrgOffset,
                                           ViReal64 *mdlAcc, ViReal64 *pkMdlAcc,
                                           ViInt16 *slot)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPTAU?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", tausec) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPTAUCHIP?", 11, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", tauchip) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPCFER?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", crrFrqErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPIQOFS?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", iqOrgOffset) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPMOD?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", mdlAcc) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPPKMOD?", 9, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", pkMdlAcc) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPSLOT?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%d[b2]", slot) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt62_status;
}

/*===========================================================================*/
/* Function: 3GPP Graph Data				                                 */
/* Purpose:  This function returns the graphic data of the 3GPP measurement. */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_data_graph3GPP (ViSession instrumentHandle,
                                          ViInt16 grphSlct,
                                          ViInt16 *iDataSize, ViReal64 iData[],
                                          ViInt16 *qDataSize, ViReal64 qData[],
                                          ViInt16 *xDataSize, ViInt16  xData[],
                                          ViInt16 *yDataSize, ViReal64 yData[])
{
    if (avopt62_invalidViInt16Range (grphSlct, 0, 9))
        return VI_ERROR_PARAMETER2;
        
    avopt62_selectGraphType3GPP (instrumentHandle, grphSlct, 0);
    Delay (0.7);
    
    return (avopt62_dataGraph (instrumentHandle, grphSlct, 
    						   iDataSize, iData, qDataSize, qData, 
    						   xDataSize, xData, yDataSize, yData));
}

/*===========================================================================*/
/* Function: QPSK							                                 */
/* Purpose:  This function returns the result of QPSK measurement.	 		 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_dataQPSK (ViSession instrumentHandle, 
									ViReal64 *rho, ViReal64 *crrFrqErr, 
									ViReal64 *crrFdThrough, ViReal64 *magErr, 
									ViReal64 *phsErr, ViReal64 *errVecMag, 
									ViInt16 *judge)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "QPRHO?", 6, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", rho) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  
    if ((avopt62_status = viWrite (instrumentHandle, "QPFER?", 6, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", crrFrqErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  
    if ((avopt62_status = viWrite (instrumentHandle, "QPIQOFS?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", crrFdThrough) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  
    if ((avopt62_status = viWrite (instrumentHandle, "QPMAG?", 6, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", magErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  
    if ((avopt62_status = viWrite (instrumentHandle, "QPPHSE?", 7, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", phsErr) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  
    if ((avopt62_status = viWrite (instrumentHandle, "QPMOD?", 6, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%f", errVecMag) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  
    if ((avopt62_status = viWrite (instrumentHandle, "QPJDG?", 6, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%d[b2]", judge) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  
    return avopt62_status;
}

/*===========================================================================*/
/* Function: QPSK Graph Data				                                 */
/* Purpose:  This function returns the graphic data of the QPSK measurement. */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_data_graphQPSK (ViSession instrumentHandle,
                                          ViInt16 grphSlct,
                                          ViInt16 *iDataSize, ViReal64 iData[],
                                          ViInt16 *qDataSize, ViReal64 qData[],
                                          ViInt16 *xDataSize, ViInt16  xData[],
                                          ViInt16 *yDataSize, ViReal64 yData[])
{
    if (avopt62_invalidViInt16Range (grphSlct, 0, 9))
        return VI_ERROR_PARAMETER2;
        
    avopt62_selectGraphTypeQPSK (instrumentHandle, grphSlct, 0);
    Delay (0.7);
    
    return (avopt62_dataGraph (instrumentHandle, grphSlct, 
    						   iDataSize, iData, qDataSize, qData, 
    						   xDataSize, xData, yDataSize, yData));
}

/*===========================================================================*/
/* Function: TX Power                                                        */
/* Purpose:  This function returns the result of TX power measurement. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_dataTXPow (ViSession instrumentHandle,
                                     ViReal64 *powerLeveldBm,
                                     ViReal64 *powerLevelW,
                                     ViReal64 *peakFactordB)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "TXPWR?", 6, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f", powerLeveldBm, powerLevelW,peakFactordB) < 3)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt62_status;
}

/*===========================================================================*/
/* Function: Power vs Time                                                   */
/* Purpose:  This function returns the result of power vs time measurement.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_dataPwrTime (ViSession instrumentHandle,
                                       ViInt16 pwrMkr1, ViInt16 pwrMkr2,
                                       ViReal64 *pkPwr1, ViReal64 *avgPwr1,
                                       ViReal64 *pkPwr2, ViReal64 *avgPwr2,
                                       ViReal64 *ratio)
{
    ViUInt32 retCnt = 0;
    ViChar buffer[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (avopt62_invalidViInt16Range (pwrMkr1, 1, 62))
       	return VI_ERROR_PARAMETER2;
    if (avopt62_invalidViInt16Range (pwrMkr2, 2, 62))
       	return VI_ERROR_PARAMETER3;
       	
    Fmt (buffer, "%s<3GPPTMKR2 %d; 3GPPTMKR1 %d", pwrMkr2, pwrMkr1);
    if ((avopt62_status = viWrite (instrumentHandle, buffer, NumFmtdBytes(), &retCnt)) < 0)
        return avopt62_status;
    
    if ((avopt62_status = viWrite (instrumentHandle, "3GPPTMK1PKF?", 12, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, buffer, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (buffer, "%s>%f", pkPwr1) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  
    if ((avopt62_status = viWrite (instrumentHandle, "3GPPTMK1AVG?", 12, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, buffer, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (buffer, "%s>%f", avgPwr1) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  
    if ((avopt62_status = viWrite (instrumentHandle, "3GPPTMK2PKF?", 12, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, buffer, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (buffer, "%s>%f", pkPwr2) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  
    if ((avopt62_status = viWrite (instrumentHandle, "3GPPTMK2AVG?", 12, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, buffer, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (buffer, "%s>%f", avgPwr2) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  
    if ((avopt62_status = viWrite (instrumentHandle, "3GPPTMKRATIO?", 13, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, buffer, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (buffer, "%s>%f", ratio) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  
	return avopt62_status;
}

/*===========================================================================*/
/* Function: CCDF			                                                 */
/* Purpose:  This function returns the result of CCDF measurement.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt62_dataCCDF (ViSession instrumentHandle,
                                    ViReal64 *pkFact, ViReal64 *avgPwr,
                                    ViReal64 *pwr10, ViReal64 *pwr1,
                                    ViReal64 *pwr01, ViReal64 *pwr001,
                                    ViReal64 *pwr0001, ViReal64 *pwr00001)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viWrite (instrumentHandle, "3GPCCDF?", 8, &retCnt)) < 0)
        return avopt62_status;
    if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f,%f,%f,%f,%f,%f", pkFact, avgPwr, pwr10, pwr1,
   			  pwr01, pwr001, pwr0001, pwr00001) < 8)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt62_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus CVIFUNC avopt62_close (ViSession instrSession)
{
    avopt62_instrRange instrPtr;
    ViSession rmSession;
    ViStatus avopt62_status = VI_SUCCESS;

    if ((avopt62_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return avopt62_status;
    if ((avopt62_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return avopt62_status;
            
    if (instrPtr != NULL) 
        free (instrPtr);
    
    avopt62_status = viClose (instrSession);
    viClose (rmSession);

    return avopt62_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean avopt62_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt62_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt62_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Unsigned Char Value Out Of Range - ViUInt8                      */
/* Purpose:  This function checks an unsigned char value to see if it        */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt62_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt62_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt62_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avopt62_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avopt62_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*================================================================================*/
/* Function: Wait On VISA Event                                                   */
/* Purpose:  This function waits for the occurrence of an enabled VISA Event. The */
/*           event type must be enabled before entering this function. Any        */
/*           programmatic commands, actions, or conditions, necessary to generate */
/*           the specified event type, must be satisified before entering this    */
/*           function. The function will wait for the specified timeout and then  */
/*           return. If the specified event is received then the function will    */
/*           return VI_SUCCESS (0), otherwise the status code for the generated   */
/*           error will be returned. In either case the function will disable the */
/*           event type and deallocate the event handle passed from viWaitOnEvent */
/*           before returning. WARNING: If VI_TMO_INFINITE is passed in as the    */
/*           timeout this function WILL NOT return until the specified event is   */
/*           received, if the event is not received the function will not return  */
/*           and it will be necessary to terminate CVI in order to regain control.*/
/*================================================================================*/
ViStatus avopt62_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn, 
            ViUInt32 timeoutIn, ViPUInt16 STB)
{            
    ViStatus avopt62_status = VI_SUCCESS, tempStatus = VI_SUCCESS;
    ViEventType eventTypeOut = 0;
    ViEvent eventHandle = 0;
    
    /* For debug purposes we want to be able to see the status returned by        */
    /* viDisableEvent and viClose if one of the internal functions fails but do   */
    /* not want to return that value as that is not where the initial error       */
    /* occurs in the function, so we assign it to tempStatus.                     */
    
    if ((avopt62_status = viWaitOnEvent (instrSession, eventTypeIn, timeoutIn, 
                                        &eventTypeOut, &eventHandle)) < 0) {
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return avopt62_status;
    } 
    
    if (eventTypeIn == VI_EVENT_SERVICE_REQ)
        if ((avopt62_status = viReadSTB (instrSession, STB)) < 0) {
            tempStatus = viClose (eventHandle);
            tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
            return avopt62_status;
        }   
        
    if ((avopt62_status = viClose (eventHandle)) < 0) {             
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return avopt62_status;                 
    }
    
    if ((avopt62_status = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE)) < 0)
        return avopt62_status;                                                   

    return avopt62_status;                                    
}    

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the avopt62_init function.  When  */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from avopt62_init to      */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus avopt62_initCleanUp (ViSession openRMSession,
                ViPSession openInstrSession, ViStatus currentStatus)
{
    avopt62_instrRange instrPtr;
    
    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL) 
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the avopt62_reset operation and by the    */
/*           avopt62_init function if the reset option has not been          */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus avopt62_defaultInstrSetup (ViSession instrSession)
{
    ViStatus avopt62_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    avopt62_instrRange instrPtr;
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    if (avopt62_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
        return avopt62_status;
    
    if (instrPtr == NULL) 
        instrPtr = malloc (sizeof (struct avopt62_statusDataRanges));

    instrPtr -> traceRange = 1001;
    instrPtr -> harmonics = 3;
    Fmt (instrPtr -> instrDriverRevision, "%s<%s", avopt62_REVISION);
    
    if ((avopt62_status = viSetAttribute (instrSession, VI_ATTR_USER_DATA, 
                            (ViUInt32)instrPtr)) < 0)
        return avopt62_status;                                       

    return avopt62_status;
}

/*===========================================================================*/
/* Function: Read Graph Data				                                 */
/* Purpose:  This function returns the graph data of the 3GPP 				 */
/*			 and QPSK measurement.											 */
/*===========================================================================*/

ViStatus avopt62_readDoubleArray (ViSession instrumentHandle, ViChar *command,
								  ViInt16 *dataSize, ViReal64 data[])
{
	ViInt16  n;
	ViUInt32 retCnt = 0;
    ViChar   rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

	if ((avopt62_status = viWrite (instrumentHandle, command, StringLength (command), &retCnt)) < 0)
       	return avopt62_status;
	if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%d[b2]", dataSize) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   			
   	n = *dataSize;
   	while (--n>=0)
   	{
	   	if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt62_status;
   		if (Scan (rdBuf, "%s>%f", data++) < 1)
   			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
	
	return avopt62_status;
}

ViStatus avopt62_readShortArray (ViSession instrumentHandle, ViChar *command,
								 ViInt16 *dataSize, ViInt16 data[])
{
	ViInt16  n;
	ViUInt32 retCnt = 0;
    ViChar   rdBuf[BUFFER_SIZE];
    ViStatus avopt62_status = VI_SUCCESS;

	if ((avopt62_status = viWrite (instrumentHandle, command, StringLength (command), &retCnt)) < 0)
       	return avopt62_status;
	if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt62_status;
   	if (Scan (rdBuf, "%s>%d[b2]", dataSize) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   			
   	n = *dataSize;
   	while (--n>=0)
   	{
	   	if ((avopt62_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt62_status;
   		if (Scan (rdBuf, "%s>%d[b2]", data++) < 1)
   			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
	
	return avopt62_status;
}

ViStatus avopt62_dataGraph (ViSession instrumentHandle, ViInt16 grphSlct,
                            ViInt16 *iDataSize, ViReal64 iData[],
                            ViInt16 *qDataSize, ViReal64 qData[],
                            ViInt16 *xDataSize, ViInt16  xData[],
                            ViInt16 *yDataSize, ViReal64 yData[])
{
    ViStatus avopt62_status = VI_SUCCESS;
    
    if (grphSlct<=6)
    {
    
    	if ((avopt62_status = avopt62_readDoubleArray (instrumentHandle, "GPHI?", iDataSize, iData)) < 0)
    		return avopt62_status;
    		
    	if ((avopt62_status = avopt62_readDoubleArray (instrumentHandle, "GPHQ?", qDataSize, qData)) < 0)
    		return avopt62_status;
    }
    
    else
    {
    	*iDataSize = 0;
    	*qDataSize = 0;
    }
    
    if (grphSlct>=4)
    {
    	if ((avopt62_status = avopt62_readShortArray (instrumentHandle, "GPHX?", xDataSize, xData)) < 0)
    		return avopt62_status;
    }
    
    else
    	*xDataSize = 0;
	
    if (grphSlct>=7)
    {
    	if ((avopt62_status = avopt62_readDoubleArray (instrumentHandle, "GPHY?", yDataSize, yData)) < 0)
    		return avopt62_status;
    }
    
    else
    	*yDataSize = 0;
	
	return avopt62_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
