#include <cvidef.h>

/*= R3267 Series - Modulation Spectrum Analyzes Include File ================*/

/*===========================================================================*/
/*  Please do not use global variables or arrays in the include file of      */
/*  instrument drivers that will be submitted for inclusion into the         */
/*  LabWindows Instrument Driver Library.                                    */
/*===========================================================================*/
     
#ifndef __avopt62_HEADER
#define __avopt62_HEADER

#include <vpptype.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************/
/*= Instrument Driver Specific Error/Warning Codes ==========================*/
/*****************************************************************************/
#define VI_ERROR_INSTR_FILE_OPEN                (_VI_ERROR+0x3FFC0800L)
#define VI_ERROR_INSTR_FILE_WRITE               (_VI_ERROR+0x3FFC0801L)
#define VI_ERROR_INSTR_INTERPRETING_RESPONSE    (_VI_ERROR+0x3FFC0803L)
#define VI_ERROR_INSTR_PARAMETER9               (_VI_ERROR+0x3FFC0809L)
#define VI_ERROR_INSTR_PARAMETER10              (_VI_ERROR+0x3FFC080AL)
#define VI_ERROR_INSTR_PARAMETER11              (_VI_ERROR+0x3FFC080BL)
#define VI_ERROR_INSTR_PARAMETER12              (_VI_ERROR+0x3FFC080CL)
#define VI_ERROR_INSTR_PARAMETER13              (_VI_ERROR+0x3FFC080DL)
#define VI_ERROR_INSTR_PARAMETER14              (_VI_ERROR+0x3FFC080EL)
#define VI_ERROR_INSTR_PARAMETER15              (_VI_ERROR+0x3FFC080FL)
#define VI_ERROR_INSTR_PARAMETER16              (_VI_ERROR+0x3FFC0810L)
#define VI_ERROR_INSTR_PARAMETER17              (_VI_ERROR+0x3FFC0811L)  
/*****************************************************************************/
/*= Define Instrument Specific Error/Warning Codes Here =====================*/
/*****************************************************************************/
#define VI_WARNING_INSTR_OFFSET                           (0x3FFC0900L)
#define VI_ERROR_INSTR_OFFSET                   (_VI_ERROR+0x3FFC0900L)

#define avopt62_ERROR_INVALID_CONFIGURATION      (VI_ERROR_INSTR_OFFSET + 0xF0L)

/*****************************************************************************/
/*= GLOBAL USER-CALLABLE FUNCTION DECLARATIONS (Exportable Functions) =======*/
/*****************************************************************************/
ViStatus _VI_FUNC avopt62_confStdSet (ViSession instrumentHandle, ViBoolean measMode,
                                      ViBoolean link, ViReal64 offsetLevel,
                                      ViBoolean frequencyInput, ViBoolean input,
                                      ViBoolean baseBandInput,
                                      ViBoolean contAutoLevel);

ViStatus CVIFUNC avopt62_ChEdt (ViSession instrumentHandle, ViBoolean settoSTD,
                                ViInt16 editChannel, ViInt16 startCHNum,
                                ViInt16 stopCHNum, ViReal64 baseFreqHz,
                                ViReal64 channelSpaceHz, ViInt16 CHOffset);

ViStatus CVIFUNC avopt62_ch_Enable (ViSession instrumentHandle,
                                    ViBoolean channel1, ViBoolean channel2,
                                    ViBoolean channel3);

ViStatus CVIFUNC avopt62_confTrgConTDP (ViSession instrumentHandle,
                                        ViInt16 triggerSource,
                                        ViBoolean trgSlope, ViInt16 triggerLevel,
                                        ViInt16 triggerPosition,
                                        ViReal64 delayTimes);

ViStatus CVIFUNC avopt62_windowSetTDP (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViBoolean window,
                                       ViReal64 winPosS, ViReal64 winWidS);

ViStatus CVIFUNC avopt62_template_Set_TDP (ViSession instrumentHandle,
                                           ViBoolean template, ViReal64 shiftXS,
                                           ViReal64 shiftYdB);

ViStatus CVIFUNC avopt62_template_editTDP (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean templateSelect,
                                           ViReal64 templateTimes,
                                           ViReal64 templateLevel);

ViStatus CVIFUNC avopt62_parameter_setTDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean TMPLCoupleofPWR,
                                           ViReal64 templateLimit,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit);

ViStatus CVIFUNC avopt62_trigger_setOOR (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes);

ViStatus CVIFUNC avopt62_window_setOOR (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 onPositionS, ViReal64 onWidthS,
                                        ViReal64 offPositionS,
                                        ViReal64 offWidthS);

ViStatus CVIFUNC avopt62_param_setOOR (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViInt16 displayUnit, ViBoolean judgement,
                                       ViReal64 upperLimit);

ViStatus CVIFUNC avopt62_trigger_setTDS (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes);

ViStatus CVIFUNC avopt62_table_editTDS (ViSession instrumentHandle,
                                        ViInt16 operation, ViInt16 tableNumber,
                                        ViBoolean freqType, ViReal64 carrierFreq,
                                        ViReal64 frequency, ViReal64 level);

ViStatus CVIFUNC avopt62_param_setTDS (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViBoolean result, ViReal64 peakMKRYDelta, 
                                       ViReal64 multiplier, ViInt16 displayUnit,
                                       ViBoolean judgement, ViBoolean preselecter);

ViStatus CVIFUNC avopt62_confGateTrg (ViSession instrumentHandle,
                                      ViBoolean gateSource, ViBoolean settoSTD,
                                      ViBoolean measMode, ViReal64 gatePosition,
                                      ViReal64 gateWidth, ViInt16 trgSource,
                                      ViInt16 triggerPosition, ViReal64 delayTime,
                                      ViBoolean trgSlope, ViInt16 triggerLevel,
                                      ViBoolean gatedSweep, ViInt16 detector);

ViStatus CVIFUNC avopt62_window_setFDP (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 positionHz, ViReal64 widthHz);

ViStatus CVIFUNC avopt62_parameter_setFDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean gatedSweep,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit);

ViStatus CVIFUNC avopt62_parameter_setOBW (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViReal64 OBW,
                                           ViInt16 detector, ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit);

ViStatus CVIFUNC avopt62_parameter_setDT (ViSession instrumentHandle,
                                          ViBoolean settoSTD, ViInt16 detector,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViReal64 templateLimit,
                                          ViBoolean judgement,
                                          ViBoolean freqSetting, ViInt16 result,
                                          ViBoolean refPower,
                                          ViReal64 symbolRate, ViReal64 rolloff);

ViStatus CVIFUNC avopt62_template_setDT (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz);

ViStatus CVIFUNC avopt62_template_editDT (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB);

ViStatus CVIFUNC avopt62_marker_editDT (ViSession instrumentHandle,
                                        ViInt16 operation,
                                        ViInt16 refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB);

ViStatus CVIFUNC avopt62_parameter_setDM (ViSession instrumentHandle,
                                          ViBoolean settoSTD,
                                          ViBoolean freqSetting,
                                          ViInt16 detector, ViInt16 result,
                                          ViBoolean refPower,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViBoolean judgement,
                                          ViReal64 templateLimit,
                                          ViReal64 symbolRate, ViReal64 rolloff);

ViStatus CVIFUNC avopt62_template_setDM (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz);

ViStatus CVIFUNC avopt62_template_editDM (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB);

ViStatus CVIFUNC avopt62_marker_editDM (ViSession instrumentHandle,
                                        ViInt16 operation,
                                        ViInt16 refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB);

ViStatus CVIFUNC avopt62_parameter_setINSPR (ViSession instrumentHandle,
                                             ViBoolean settoSTD,
                                             ViBoolean freqSetting,
                                             ViInt16 detector,
                                             ViBoolean refPower,
                                             ViReal64 peakMkrYDelta,
                                             ViInt16 result, ViInt16 displayUnit,
                                             ViBoolean TMPLCoupleofPWR,
                                             ViReal64 templateLimit,
                                             ViBoolean judgement);

ViStatus CVIFUNC avopt62_template_setINSPR (ViSession instrumentHandle,
                                            ViBoolean template,
                                            ViReal64 shiftXHz, ViReal64 shiftYdB,
                                            ViReal64 deltaXHz);

ViStatus CVIFUNC avopt62_template_editINSPR (ViSession instrumentHandle,
                                             ViInt16 operation,
                                             ViReal64 templateFreqHz,
                                             ViReal64 templateLeveldB);

ViStatus CVIFUNC avopt62_marker_editINSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean refMarkerType,
                                           ViReal64 refBandWidthHz,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz,
                                           ViReal64 limitPowerdB);

ViStatus CVIFUNC avopt62_parameter_setOUTSPR (ViSession instrumentHandle,
                                              ViBoolean settoSTD,
                                              ViInt16 detector,
                                              ViReal64 peakMkrYDelta,
                                              ViInt16 displayUnit,
                                              ViBoolean judgement,
                                              ViBoolean preselector);

ViStatus CVIFUNC avopt62_table_setOUTSPR (ViSession instrumentHandle,
                                          ViInt16 tableNumber,
                                          ViBoolean loadTable,
                                          ViBoolean saveTable, ViInt16 tableSize,
                                          ViReal64 startFreqHz[],
                                          ViReal64 stopFreqHz[], ViReal64 RBW[],
                                          ViReal64 VBW[], ViReal64 sweepTime[],
                                          ViReal64 limitLevel[]);

ViStatus CVIFUNC avopt62_table_editOUTSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViInt16 tableNumber,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz, ViReal64 RBW,
                                           ViReal64 VBW, ViReal64 sweepTime,
                                           ViReal64 limitLevel);

ViStatus CVIFUNC avopt62_confYCodeDomPow (ViSession instrumentHandle,
                                          ViBoolean tauOffset,
                                          ViReal64 tauOffsetValus,
                                          ViBoolean equalFilter,
                                          ViBoolean measMode,
                                          ViReal64 thresholddB,
                                          ViInt16 measRangechip,
                                          ViBoolean PNSearchMode,
                                          ViInt16 PNOffset, ViBoolean trgSlope,
                                          ViReal64 extTrigDelaychip,
                                          ViInt16 pagingChannel,
                                          ViInt16 trafficChannel,
                                          ViBoolean phaseUnit,
                                          ViBoolean powerUnit,
                                          ViBoolean freqMeasRange,
                                          ViInt16 triggerSource);

ViStatus _VI_FUNC avopt62_parameter_setDownSlot3GPP (ViSession instrumentHandle,
                                                     ViBoolean scramblingCodeDefine,
                                                     ViInt32 scramblingCodeNo,
                                                     ViBoolean triggerMode,
                                                     ViBoolean EXTTriggerSlope,
                                                     ViReal64 EXTTriggerDelay,
                                                     ViBoolean searchMode,
                                                     ViInt16 primaryCPICHSF,
                                                     ViInt16 primaryCPICHNo,
                                                     ViInt16 activeCHDetection,
                                                     ViInt16 analysisRate,
                                                     ViInt16 measUnit,
                                                     ViInt16 measStartPosition,
                                                     ViInt16 threshold,
                                                     ViBoolean phaseInverse,
                                                     ViBoolean frequencyError,
                                                     ViBoolean transmitTiming);

ViStatus _VI_FUNC avopt62_parameter_setDownFrame3GPP (ViSession instrumentHandle,
                                                      ViBoolean scramblingCodeDefine,
                                                      ViInt32 scramblingCodeNo,
                                                      ViBoolean triggerMode,
                                                      ViBoolean EXTTriggerSlope,
                                                      ViReal64 EXTTriggerDelay,
                                                      ViBoolean searchMode,
                                                      ViInt16 primaryCPICHSF,
                                                      ViInt16 primaryCPICHNo,
                                                      ViInt16 activeCHDetection,
                                                      ViInt16 measChannelSF,
                                                      ViInt16 measChannelNo,
                                                      ViInt16 measSlots,
                                                      ViInt16 measStartPosition,
                                                      ViInt16 threshold,
                                                      ViBoolean phaseInverse);

ViStatus _VI_FUNC avopt62_parameter_setUpSlot3GPP (ViSession instrumentHandle,
                                                   ViInt32 scramblingCodeNo,
                                                   ViBoolean triggerMode,
                                                   ViBoolean EXTTriggerSlope,
                                                   ViReal64 EXTTriggerDelay,
                                                   ViInt16 DPCCHSF,
                                                   ViInt16 DPCCHNo,
                                                   ViInt16 analysisRate,
                                                   ViInt16 measUnit,
                                                   ViInt16 measStartPosition,
                                                   ViInt16 threshold,
                                                   ViBoolean phaseInverse,
                                                   ViBoolean frequencyError);

ViStatus _VI_FUNC avopt62_parameter_setUpFrame3GPP (ViSession instrumentHandle,
                                                    ViInt32 scramblingCodeNo,
                                                    ViBoolean triggerMode,
                                                    ViBoolean EXTTriggerSlope,
                                                    ViReal64 EXTTriggerDelay,
                                                    ViInt16 DPCCHSF,
                                                    ViInt16 DPCCHNo,
                                                    ViInt16 measChannelSF,
                                                    ViInt16 measChannelNo,
                                                    ViBoolean measBranch,    
                                                    ViInt16 measSlots,
                                                    ViInt16 measStartPosition,
                                                    ViInt16 threshold,
                                                    ViBoolean phaseInverse);

ViStatus _VI_FUNC avopt62_tableSize_set3GPP (ViSession instrumentHandle,
                                             ViInt16 multiChannelNo);

ViStatus _VI_FUNC avopt62_table_edit3GPP (ViSession instrumentHandle,
                                          ViInt16 editChannelNo, ViInt16 SF,
                                          ViInt16 number);

ViStatus _VI_FUNC avopt62_parameter_setQPSK (ViSession instrumentHandle,
                                             ViBoolean rootNyquist,
                                             ViInt16 measRange,
                                             ViBoolean triggerMode,
                                             ViBoolean EXTTriggerSlope,
                                             ViReal64 EXTTriggerDelay,
                                             ViBoolean judgement,
                                             ViReal64 limitRho);

ViStatus _VI_FUNC avopt62_parameter_setTxPower (ViSession instrumentHandle,
                                                ViBoolean rootNyquist,
                                                ViBoolean triggerMode,
                                                ViBoolean EXTTriggerSlope,
                                                ViReal64 EXTTriggerDelay);

ViStatus _VI_FUNC avopt62_scale_setPwrTime (ViSession instrumentHandle,
                                            ViInt16 YScaleUpper,
                                            ViInt16 YScaleRange,
                                            ViBoolean powerUnit);

ViStatus _VI_FUNC avopt62_parameter_setPwrTime (ViSession instrumentHandle,
                                                ViBoolean rootNyquist,
                                                ViInt16 triggerMode,
                                                ViBoolean triggerSlope,
                                                ViInt16 triggerLevel,
                                                ViReal64 triggerDelay,
                                                ViInt16 measLength,
                                                ViBoolean graphPlotType,
                                                ViBoolean omitTransientSection_forAVG);

ViStatus _VI_FUNC avopt62_scale_setCCDF (ViSession instrumentHandle,
                                         ViInt16 XScaleMax, ViInt16 XScaleRange,
                                         ViBoolean powerUnit);

ViStatus _VI_FUNC avopt62_parameter_setCCDF (ViSession instrumentHandle,
                                             ViBoolean rootNyquist,
                                             ViBoolean triggerMode,
                                             ViBoolean triggerSlope,
                                             ViReal64 triggerDelay,
                                             ViInt32 measLength);

ViStatus CVIFUNC avopt62_average_set (ViSession instrSession, ViInt16 function,
                ViInt16 AVGTimes);

ViStatus CVIFUNC avopt62_execute_auto_lvl (ViSession instrumentHandle,
                                           ViInt16 function);

ViStatus CVIFUNC avopt62_actYDispCtrl (ViSession instrSession, ViInt16 zoomingFunction,
				ViReal64 windowPositions, ViReal64 windowWidths, ViBoolean verticalZoom,
				ViInt16 YScale);

ViStatus _VI_FUNC avopt62_act_DCCal (ViSession instrumentHandle);

ViStatus CVIFUNC avopt62_actMeasStart (ViSession instrumentHandle, ViInt16 function);

ViStatus _VI_FUNC avopt62_selectGraphType3GPP (ViSession instrumentHandle,
                                               ViInt16 graphSelection,
                                               ViInt16 displayStart);

ViStatus _VI_FUNC avopt62_selectGraphTypeQPSK (ViSession instrumentHandle,
                                               ViInt16 graphSelection,
                                               ViInt16 displayStart);

ViStatus CVIFUNC avopt62_actGraphMkr (ViSession instrSession,
                ViBoolean marker);

ViStatus CVIFUNC avopt62_actYLimitLine (ViSession instrSession,
                ViInt16 typeSelection, ViBoolean tableType);
          
ViStatus CVIFUNC avopt62_check_measMode (ViSession instrumentHandle,
                                         ViInt16 *returnmode);

ViStatus CVIFUNC avopt62_data_TDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement);

ViStatus CVIFUNC avopt62_dataOnOffRatio (ViSession instrumentHandle,
                                          ViReal64 *ONLevel, ViReal64 *OFFLevel,
                                          ViReal64 *ONOFFRaito,
                                          ViInt16 *judgement);

ViStatus CVIFUNC avopt62_data_spuriTD (ViSession instrumentHandle,
                                       ViInt16 *tableSize,
                                       ViReal64 frequencyHz[], ViReal64 level[],
                                       ViInt16 judgement[]);

ViStatus CVIFUNC avopt62_data_FDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement);

ViStatus CVIFUNC avopt62_dataOBW (ViSession instrSession, ViPReal64 OBWHz,
                				  ViPReal64 lowFreqHz, ViPReal64 hiFreqHz,
                				  ViInt16 *judgement);

ViStatus CVIFUNC avopt62_data_duetoTRN (ViSession instrumentHandle,
                                        ViInt16 *numberofACP,
                                        ViReal64 ACPDatadBc[],
                                        ViInt16 judgement[]);

ViStatus CVIFUNC avopt62_data_duetoMOD (ViSession instrumentHandle,
                                        ViInt16 *numberofACP, ViReal64 ACPdBc[],
                                        ViInt16 ACPJudge[]);

ViStatus CVIFUNC avopt62_dataSpurMeas (ViSession instrumentHandle,
                                       ViInt16 *numberofPoint,
                                       ViReal64 frequencyHz[],
                                       ViReal64 leveldBm[], ViInt16 judge[]);

ViStatus CVIFUNC avopt62_data_spuriOB (ViSession instrumentHandle,
                                       ViInt16 *numberofPoint,
                                       ViReal64 frequencyHz[],
                                       ViReal64 leveldBm[], ViInt16 judgement[]);

ViStatus _VI_FUNC avopt62_dataDownSlot3GPP (ViSession instrumentHandle,
                                            ViReal64 *rho, ViReal64 *tausec,
                                            ViReal64 *tauchip,
                                            ViReal64 *carrierFrequencyErrorHz,
                                            ViReal64 *IQOriginOffsetdBc,
                                            ViReal64 *magnitude_error_rms,
                                            ViReal64 *phaseErrordeg_rms,
                                            ViReal64 *modulationAccuracy_rms,
                                            ViReal64 *peakMagnitudeError,
                                            ViReal64 *peakPhaseErrordeg,
                                            ViReal64 *peakModulationAccuracy,
                                            ViInt16 *slot,
                                            ViInt16 *ScramblingCodeNo,
                                            ViInt16 *scramblingCodeGroupNo,
                                            ViReal64 *SCHPowerdB,
                                            ViReal64 *powerRatioPSCHSSCHdB,
                                            ViReal64 *peakCodeDomainErrordB);

ViStatus _VI_FUNC avopt62_dataDownFrame3GPP (ViSession instrumentHandle,
                                             ViReal64 *tausec, ViReal64 *tauchip,
                                             ViReal64 *carrierFrequencyErrorHz,
                                             ViReal64 *IQOriginOffsetdBc,
                                             ViReal64 *magnitude_error_rms,
                                             ViReal64 *phaseErrordeg_rms,
                                             ViReal64 *modulationAccuracy_rms,
                                             ViReal64 *peakMagnitudeError,
                                             ViReal64 *peakPhaseErrordeg,
                                             ViReal64 *peakModulationAccuracy,
                                             ViInt16 *slot,
                                             ViInt16 *scramblingCodeNo,
                                             ViInt16 *scramblingCodeGroupNo,
                                             ViReal64 *powerRatioPSCHSSCHdB);

ViStatus _VI_FUNC avopt62_dataUpSlot3GPP (ViSession instrumentHandle,
                                          ViReal64 *rho, ViReal64 *tausec,
                                          ViReal64 *tauchip,
                                          ViReal64 *carrierFrequencyErrorHz,
                                          ViReal64 *IQOriginOffsetdBc,
                                          ViReal64 *magnitudeError_rms,
                                          ViReal64 *phaseErrordeg_rms,
                                          ViReal64 *modulationAccuracy_rms,
                                          ViReal64 *peakMagnitudeError,
                                          ViReal64 *peakPhaseErrordeg,
                                          ViReal64 *peakModulationAccuracy,
                                          ViInt16 *slot,
                                          ViReal64 *peakCodeDomainErrordB);

ViStatus _VI_FUNC avopt62_dataUpFrame3GPP (ViSession instrumentHandle,
                                           ViReal64 *tausec, ViReal64 *tauchip,
                                           ViReal64 *carrierFrequencyErrorHz,
                                           ViReal64 *IQOriginOffsetdBc,
                                           ViReal64 *modulationAccuracy_rms,
                                           ViReal64 *peakModulationAccuracy,
                                           ViInt16 *slot);

ViStatus _VI_FUNC avopt62_data_graph3GPP (ViSession instrumentHandle,
                                          ViInt16 graphSelection,
                                          ViInt16 *IPhaseDataSize,
                                          ViReal64 IPhaseData[],
                                          ViInt16 *QPhaseDataSize,
                                          ViReal64 QPhaseData[],
                                          ViInt16 *XAxisDataSize,
                                          ViInt16  XAxisData[],
                                          ViInt16 *YAxisDataSize,
                                          ViReal64 YAxisData[]);

ViStatus _VI_FUNC avopt62_dataQPSK (ViSession instrumentHandle, ViReal64 *rho,
                                    ViReal64 *carrierFrequencyErrorHz,
                                    ViReal64 *carrierFeedthroughdBc,
                                    ViReal64 *magnitudeError_rms,
                                    ViReal64 *phaseErrordeg_rms,
                                    ViReal64 *errorVectorMagnitude_rms,
                                    ViInt16 *judgement);

ViStatus _VI_FUNC avopt62_data_graphQPSK (ViSession instrumentHandle,
                                          ViInt16 graphSelection,
                                          ViInt16 *IPhaseDataSize,
                                          ViReal64 IPhaseData[],
                                          ViInt16 *QPhaseDataSize,
                                          ViReal64 QPhaseData[],
                                          ViInt16 *XAxisDataSize,
                                          ViInt16  XAxisData[],
                                          ViInt16 *YAxisDataSize,
                                          ViReal64 YAxisData[]);

ViStatus CVIFUNC avopt62_dataTXPow (ViSession instrumentHandle,
                                     ViReal64 *powerLeveldBm,
                                     ViReal64 *powerLevelW,
                                     ViReal64 *peakFactordB);

ViStatus _VI_FUNC avopt62_dataPwrTime (ViSession instrumentHandle,
                                       ViInt16 powerMarker1,
                                       ViInt16 powerMarker2,
                                       ViReal64 *_1PeakFactor,
                                       ViReal64 *_1AveragePower,
                                       ViReal64 *_2PeakFactor,
                                       ViReal64 *_2AveragePower,
                                       ViReal64 *powerMarkerRatio);

ViStatus _VI_FUNC avopt62_dataCCDF (ViSession instrumentHandle,
                                    ViReal64 *peakFactor, ViReal64 *averagePower,
                                    ViReal64 *_10, ViReal64 *_1, ViReal64 *_01,
                                    ViReal64 *_001, ViReal64 *_0001,
                                    ViReal64 *_00001);

ViStatus CVIFUNC avopt62_close (ViSession instrSession);

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif

/*****************************************************************************/
/*=== END INCLUDE FILE ======================================================*/
/*****************************************************************************/

#endif
